# -*- coding: utf-8 -*-
# (C) Copyright (C) 2012, 2013 Pablo Angulo
# This file is part of karakolas <karakolas.org>.

# karakolas is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as 
# published by the Free Software Foundation, either version 3 of the 
# License, or (at your option) any later version.

# karakolas is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU Affero General Public 
# License along with karakolas.  If not, see 
# <http://www.gnu.org/licenses/>.


#########################################################################
## 
#########################################################################

import os
import os.path
import subprocess
from encodings import codecs
import datetime
from collections import defaultdict
import re

CONTACTA_ADMIN  = 'Contacta con el administrador: karakolas@cancamusa.net'

def default_action(g):
    s,k = g.role.split('_')
    id = int(k)
    if s == 'admins':
        return URL(c='gestion_pedidos', f='gestion_pedidos', vars=dict(grupo=id))
    if s == 'red':
        return URL(c='redes', f='admin_red', vars=dict(grupo=id))
    if s == 'miembros':
        return URL(c='peticion', f='vista_pedidos', vars=dict(grupo=id))    
    
def index():
    if auth.user:
        redirect(URL(f='home'))
    else:
        slug = request.vars.slug or 'welcome'
        p = db.paginas_especiales(slug=slug)
        return dict(page=p)

@auth.requires_membership('webadmins')
def edita():
    record = db.paginas_especiales(slug=request.args[0])
    if not record:
        rid = db.paginas_especiales.insert(slug=request.args[0])
        record = db.paginas_especiales(rid)
    form = SQLFORM(db.paginas_especiales, record)
    if form.process().accepted:
        response.flash = T('Texto de %s actualizado'%request.args[0])
    elif form.errors:
        response.flash = T('El formulario de nuevo productor tiene errores, Por favor revisalos.')
    return dict(form = form, record=record)

@auth.requires_login()
def none():
    '''Necesario para que funcione el borrado de filas en las tablas
    '''
    return dict()

@auth.requires_login()
def home():
    id = auth.user.id
    grupos = db((db.auth_membership.user_id==id) &
                (db.auth_membership.group_id==db.auth_group.id)) \
            .select(db.auth_group.ALL)
    gruposa = [g for g in grupos if (g.role.startswith('admins') or
                                 g.role.startswith('red') or
                                 g.role.startswith('miembros'))]
    if not gruposa:
        redirect(URL(f='listas_de_espera'))
    elif len(gruposa)==1:
        redirect(default_action(gruposa[0]))
    opciones = []
    if len(grupos)==1:
        redirect(default_action(g))
    for g in grupos:
        if g.role == 'webadmins':
            continue
        s,k = g.role.split('_')
        id = int(k)
        if s == 'user':
            continue
        if s == 'admins' or s == 'red':
#            opciones.append(
#               LI('%s: '%g.description, 
#                  A('Administrar', 
#                    _href=default_action(g))))
            opciones.append(H2(g.description))
            opciones.append(A('Administrar', 
                    _href=default_action(g) ))
        if s == 'miembros':
            opciones.append(H2(g.description ))
            contenido = LOAD('peticion','vista_pedidos.load', ajax=True, vars=dict(grupo=id))
            opciones.append(DIV(contenido, _class='cuadro'))
#            opciones.append(LI('%s: '%g.description, 
#                               A('Ver Pedidos Abiertos', _href=default_action(g))))
    return dict(acciones=DIV(*opciones))

@auth.requires_login()
def resumen():
    return dict()
    
def user():
    """
    exposes:
    http://..../[app]/default/user/login
    http://..../[app]/default/user/logout
    http://..../[app]/default/user/register
    http://..../[app]/default/user/profile
    http://..../[app]/default/user/retrieve_password
    http://..../[app]/default/user/change_password
    use @auth.requires_login()
        @auth.requires_membership('group name')
        @auth.requires_permission('read','table name',record_id)
    to decorate functions that need access control
    """
    return dict(form=auth())


def download():
    """
    allows downloading of uploaded files
    http://..../[app]/default/download/[filename]
    """
    return response.download(request,db)


def call():
    """
    exposes services. for example:
    http://..../[app]/default/call/jsonrpc
    decorate with @services.jsonrpc the functions to expose
    supports xml, json, xmlrpc, jsonrpc, amfrpc, rss, csv
    """
    return service()


@auth.requires_signature()
def data():
    """
    http://..../[app]/default/data/tables
    http://..../[app]/default/data/create/[table]
    http://..../[app]/default/data/read/[table]/[id]
    http://..../[app]/default/data/update/[table]/[id]
    http://..../[app]/default/data/delete/[table]/[id]
    http://..../[app]/default/data/select/[table]
    http://..../[app]/default/data/search/[table]
    but URLs must be signed, i.e. linked with
      A('table',_href=URL('data/tables',user_signature=True))
    or with the signed load operator
      LOAD('default','data.load',args='tables',ajax=True,user_signature=True)
    """
    return dict(form=crud())


#########################################################################

#@auth.requires(auth.has_membership('admins_%s'%request.vars.grupo) or 
#               auth.has_membership('webadmins'))
@auth.requires(auth.has_membership(group_id=request.vars.group_id) or 
               auth.has_membership('webadmins'))
def lista_miembros():
    gid = int(request.vars.group_id)
    for k,v in request.vars.iteritems():
        if k.startswith('del_'):
            id = int(k[4:])
            db((db.auth_membership.group_id==gid) &
               (db.auth_membership.user_id==id)).delete()
    miembros = db(db.auth_membership.group_id==gid) \
               .select(db.auth_membership.user_id)
    usuarios = [db.auth_user[row.user_id] for row in miembros]
    f = FORM(TABLE(TR(TD('usuario'), TD('Nombre'),
                      TD('Eliminar del grupo')),
                   *[TR(TD(user.username), 
                        TD('%s %s'%(user.first_name,user.last_name)),
                        TD(INPUT(_value='Eliminar', _type='checkbox', _name='del_%s'%user.id) ))
                     for user in usuarios]),
            INPUT(_type='submit', _name='submit', _value='Sacar del grupo a los miembros seleccionados'))
    return dict(lista_miembros=f, hay_miembros=bool(miembros))

@auth.requires(auth.has_membership(group_id=request.vars.group_id) or 
               auth.has_membership('webadmins'))
def crud_auth_group():
    igroup = int(request.vars.group_id)
    role   = db.auth_group(igroup).role
    description   = db.auth_group(igroup).description
    if not (role.startswith('admin') or role.startswith('red')):
        redirect(auth.settings.on_failed_authorization)
    new_member = FORM(FIELDSET(T('username del nuevo administrador: '), 
                                INPUT(_name='miembro_username')),
                       INPUT(_type='submit', _name='incluir_miembro', 
                             _value=T('Incluir miembro')))

#    new_member = SQLFORM.factory(Field('username', 
#                                widget=SQLFORM.widgets.autocomplete(
#                                        request, db.auth_user.username, 
#                                        db.auth_user.id, min_length=2)))
    if new_member.accepts(request.vars):
        usuario = db.auth_user(username=request.vars.miembro_username)
        if usuario:
            if db((db.auth_membership.group_id==igroup)&
                  (db.auth_membership.user_id==usuario.id)).count()>0:
                response.flash = 'el usuario ya está en el grupo'                
            else:
                db.auth_membership.insert(**dict(group_id=igroup, user_id=usuario.id))
                response.flash = 'ok'
        else:
            response.flash = 'Usuario desconocido'
    return dict(new_member=new_member, group=igroup, description=description)

@auth.requires_login() 
def listas_de_espera():
    lespera = db((db.grupo.red==False) & (db.grupo.lista_de_espera==True)).select(db.grupo.id)
    ls = DIV(*[LOAD('default', 'info_grupo.load', ajax=True, vars=dict(grupo=row.id))
                for row in lespera])
    return dict(lista=ls)

@auth.requires_login() 
def info_grupo():
    igrupo = int(request.vars.grupo)
    grupo  = db.grupo[igrupo]
    def info_form():
        r1 = db.personaXgrupo(grupo=igrupo, persona=auth.user_id)
        r2 = db.personaXlistaespera(grupo=igrupo, persona=auth.user_id)
        if r1 and r1.activo:
            status = T('Estás en el grupo %s')%grupo.nombre
            accion = T('Irme del grupo')
        elif r2 and r2.activo:
            status = T('Estás en la lista de espera del grupo %s')%grupo.nombre
            accion = T('Quitarme de la lista de espera')
        elif grupo.prueba:
            status = T('No estás en el grupo de prueba: %s')%grupo.nombre
            accion = T('Incorporarme al grupo')
        else:
            status = T('No estás en la lista de espera del grupo %s')%grupo.nombre
            accion = T('Incorporarme a la lista de espera')
        return FORM(H3(grupo.nombre),P(grupo.direccion), P(grupo.descripcion),
                     P(EM(status)), 
                     INPUT(_value=accion, 
                           _type='submit', _name='entrar'))
    t=info_form()
    if t.accepts(request.vars):
        r1 = db.personaXgrupo(grupo=igrupo, persona=auth.user_id)
        r2 = db.personaXlistaespera(grupo=igrupo, persona=auth.user_id)
        if r1 and r1.activo:
            r1.update_record(activo=False)
            r2.update_record(activo=False)
            imiembros = auth.id_group('miembros_%d'%igrupo)
            db((db.auth_membership.group_id==imiembros) &
               (db.auth_membership.user_id==auth.user.id)).delete()
            response.flash = 'Ya no estás en el grupo %s'%grupo.nombre
        elif r2 and r2.activo:
            r2.update_record(activo=False)
            response.flash = XML(T('Te hemos quitado de la lista de espera de %s')%grupo.nombre)
        else:
            if grupo.prueba:
                if r1:
                    r1.update_record(activo=True)
                else:
                    from random import randint
                    db.personaXgrupo.insert(grupo=igrupo, persona=auth.user_id, 
                            unidad=randint(1,10))
                imiembros = auth.id_group('miembros_%d'%igrupo)
                iadmins = auth.id_group('admins_%d'%igrupo)
                db.auth_membership.insert(group_id=imiembros, user_id=auth.user_id)
                db.auth_membership.insert(group_id=iadmins, user_id=auth.user_id)
                if not r2:
                    db.personaXlistaespera.insert(grupo=igrupo, persona=auth.user_id,
                                                  fecha_de_solicitud=request.now,
                                                  activo=False)
                response.flash = 'Te hemos incorporadol grupo %s'%grupo.nombre
            else:
                if r2:
                    r2.update_record(activo=True)
                else:
                    db.personaXlistaespera.insert(grupo=igrupo, persona=auth.user_id,
                                                  fecha_de_solicitud=request.now)
                response.flash = 'Te hemos incorporado a la lista de espera de %s'%grupo.nombre
        t=info_form()
    return dict(form=t)
    

