# -*- coding: utf-8 -*-
# (C) Copyright (C) 2012, 2013 Pablo Angulo
# This file is part of karakolas <karakolas.org>.

# karakolas is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as 
# published by the Free Software Foundation, either version 3 of the 
# License, or (at your option) any later version.

# karakolas is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU Affero General Public 
# License along with karakolas.  If not, see 
# <http://www.gnu.org/licenses/>.

# Datos estadisticos de solo lectura

import os
import os.path
import subprocess
from encodings import codecs
import datetime
from collections import defaultdict
import re
#from rsheet import ROSheet

@auth.requires_login()
def index():
    stat_global = LOAD('estadisticas', 'stat_productores_persona.load')
    stat_productor = FORM(FIELDSET(T('Ver estadisticas detalladas de: '),
                 SELECT(OPTION(T('Elige un productor'), _value=-1),
                        *[OPTION(p.nombre, _value=p.id)
                          for p in db((db.productor.id>0) &
                                      (db.productor.grupo==db.grupo.id) &
                                      (db.personaXgrupo.grupo==db.grupo.id) &
                                      (db.personaXgrupo.activo==True) &
                                      (db.personaXgrupo.persona==auth.user.id))\
                                    .select(db.productor.nombre,
                                            db.productor.id)],
                        **dict(_name='productor', _id='productor'))),
                 INPUT(_type='submit', _name='submit', 
                       _value=T('Ver estadísticas')))
    if request.vars.submit:
        redirect(URL(c='estadisticas', f='stat_productor_persona', vars={'productor':request.vars.productor}))
    
    return dict(stat_productor=stat_productor,
                 stat_global=stat_global)

@auth.requires_membership('admins_%s'%request.vars.grupo)
def stat_grupo():
    igrupo = int(request.vars.grupo)
    stat_global = LOAD('estadisticas', 'stat_productores_grupo.load', vars=dict(grupo=igrupo))
    stat_productor = FORM(FIELDSET(T('Ver estadisticas detalladas de: '),
                 SELECT(OPTION(T('Elige un productor'), _value=-1),
                        *[OPTION(p.nombre, _value=p.id)
                          for p in db((db.productor.id>0) &
                                      (db.productor.grupo==igrupo))\
                                    .select(db.productor.nombre,
                                            db.productor.id)],
                        **dict(_name='productor', _id='productor'))),
                 INPUT(_type='submit', _name='submit', 
                       _value=T('Ver estadísticas')))
    if request.vars.submit:
        redirect(URL(c='estadisticas', f='stat_productor_grupo', vars={'productor':request.vars.productor}))
    
    return dict(stat_productor=stat_productor,
                 stat_global=stat_global)

NUMBER = re.compile(r'(\d+\.\d*[1-9])0*|(\d+)')

def represent_number(d):
    if d:
        gs = NUMBER.match('%.2f'%d).groups()
        if gs:
            return gs[0] or gs[1]
        return d
    return ''

@auth.requires_membership(grupo_del_pedido(request.vars.pedido))
def stat_pedido():
    '''Muestra una tabla productos x personas con la peticion de cada persona
    de ese producto, para un pedido fijo.
    '''
    PEDIDO = request.vars.pedido
    grupo = db.pedido(PEDIDO).productor.grupo
    productos = [row.producto for row in
                 db(db.productoXpedido.pedido==PEDIDO).select(db.productoXpedido.producto)]
    def col_represent(uid):
        usuario = db.auth_user[uid]
        unidad  = db.personaXgrupo(persona=usuario, grupo=grupo).unidad
        return XML('%d<br/>%s %s'%(unidad, usuario.first_name, usuario.last_name[0]))
    def row_represent(pid):
        return db.producto[pid].nombre

    #TODO: la linea de abajo no calcula los miembros de grupos que pertenecen a 
    #la red para un pedido coordinado
    miembros  = [row.persona for row in 
                 db(db.personaXgrupo.grupo==grupo)\
                  .select(db.personaXgrupo.persona, 
                          orderby=db.personaXgrupo.unidad)]

    sheet = ROSheet((db.productoXpedido.pedido==PEDIDO) &
                    (db.item.productoXpedido==db.productoXpedido.id),
                    db.productoXpedido.producto,
                    db.item.persona,
                    db.item.cantidad,
                    col_ids=miembros,
                    row_ids=productos,
                    col_represent=col_represent,
                    row_represent=row_represent,
                    data_represent=represent_number
                    )
    return dict(sheet=sheet)

@auth.requires(tiene_acceso_al_grupo(request.vars.productor))
def stat_productor_grupo():
    '''Muestra una tabla productos x fecha_pedido para un productor fijo,
    con la cantidad total que se pide de cada producto en cada fecha
    '''
    iproductor = int(request.vars.productor)
    igrupo = db.productor[iproductor].grupo
    #especificar "productos" es posible pero innecesario
#    productos = [row.id for row in
#                 db(db.producto.productor==PRODUCTOR).select(db.producto.id)]
    def row_represent(pid):
        return db.producto[pid].nombre
    sheet = ROSheet((db.item.productoXpedido==db.productoXpedido.id) &
                    (db.productoXpedido.pedido==db.pedido.id) &
                    (db.pedido.productor==iproductor),
                    db.productoXpedido.producto,
                    db.pedido.fecha_reparto,
                    db.item.cantidad.sum(),
#                    row_ids=productos,
                    row_represent=row_represent)
    return dict(sheet=sheet,
                 igrupo=igrupo)

@auth.requires(tiene_acceso_al_grupo(request.vars.productor))
def stat_productor_grupo_bis():
    '''El resultado deberia ser similar a rosheet_test2,
    aunque al final cambia el orden de los productos 
    (orden por id o orden alfabetico)
    '''
    PRODUCTOR = request.vars.productor
    sheet = ROSheet((db.item.productoXpedido==db.productoXpedido.id) &
                    (db.productoXpedido.pedido==db.pedido.id) &
                    (db.pedido.productor==PRODUCTOR) &
                    (db.productoXpedido.producto==db.producto.id) ,
                    db.producto.nombre,
                    db.pedido.fecha_reparto,
                    db.item.cantidad.sum())
    return dict(sheet=sheet)


@auth.requires(tiene_acceso_al_grupo(request.vars.productor))
def stat_productor_persona():
    '''Muestra una tabla productos x fecha_pedido para un productor fijo,
    con la cantidad que pide una persona de cada producto en cada fecha
    '''
    iproductor = int(request.vars.productor)
    productor  = db.productor[iproductor]
    ipersona  = auth.user.id
    #especificar "productos" es posible pero innecesario
#    productos = [row.id for row in
#                 db(db.producto.productor==PRODUCTOR).select(db.producto.id)]
    def row_represent(pid):
        return db.producto[pid].nombre
    sheet = ROSheet((db.peticion.productoXpedido==db.productoXpedido.id) &
                    (db.peticion.persona==ipersona) &
                    (db.productoXpedido.pedido==db.pedido.id) &
                    (db.pedido.productor==iproductor),
                    db.productoXpedido.producto,
                    db.pedido.fecha_reparto,
                    db.peticion.cantidad,
#                    row_ids=productos,
                    row_represent=row_represent)
    return dict(sheet=sheet, nombre_productor=productor.nombre)

@auth.requires(auth.has_membership(role='admins_%s'%request.vars.grupo) or
               auth.has_membership(role='miembros_%s'%request.vars.grupo))
def stat_productores_grupo():
    '''Muestra una tabla productores x fecha_pedido con el dinero total que se mueve
    '''
    GRUPO = request.vars.grupo
    sheet = ROSheet((db.item.productoXpedido==db.productoXpedido.id) &
                    (db.productoXpedido.pedido==db.pedido.id) &
                    (db.pedido.productor==db.productor.id) &
                    (db.productor.grupo==GRUPO),
                    db.productor.nombre,
                    db.pedido.fecha_reparto,
                    db.item.precio_total.sum())
    return dict(sheet=sheet)

@auth.requires_login()
def stat_productores_persona():
    '''Muestra una tabla productores x fecha_pedido con el dinero total que se mueve
    '''
    ipersona  = auth.user.id
    sheet = ROSheet((db.peticion.productoXpedido==db.productoXpedido.id) &
                    (db.productoXpedido.pedido==db.pedido.id) &
                    (db.peticion.persona==ipersona) &
                    (db.pedido.productor==db.productor.id),
                    db.productor.nombre,
                    db.pedido.fecha_reparto,
                    db.peticion.precio_total.sum())
    return dict(sheet=sheet)


