# -*- coding: utf-8 -*-
# (C) Copyright (C) 2012, 2013 Pablo Angulo
# This file is part of karakolas <karakolas.org>.

# karakolas is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as 
# published by the Free Software Foundation, either version 3 of the 
# License, or (at your option) any later version.

# karakolas is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU Affero General Public 
# License along with karakolas.  If not, see 
# <http://www.gnu.org/licenses/>.

js_jqplot = '''<script type="text/javascript">
$(document).ready(function(){
    // Can specify a custom tick Array.
    // Ticks should match up one for each y value (category) in the series.
    var ticks = {{=XML(ticks)}};
     
    var plot1 = $.jqplot('chart1', {{=data}}, {
        stackSeries: true,
        // The "seriesDefaults" option is an options object that will
        // be applied to all series in the chart.
        seriesDefaults:{
            renderer:$.jqplot.BarRenderer,
            rendererOptions: {fillToZero: true}
        },
        // Custom labels for the series are specified with the "label"
        // option on the series option.  Here a series option object
        // is specified for each series.
        series:[
            {{=XML(series_labels)}}
        ],
        // Show the legend and put it outside the grid, but inside the
        // plot container, shrinking the grid to accomodate the legend.
        // A value of "outside" would not shrink the grid and allow
        // the legend to overflow the container.
        legend: {
            show: true,
            placement: 'outsideGrid'
        },
        axes: {
            // Use a category axis on the x axis and use our custom ticks.
            xaxis: {
                renderer: $.jqplot.CategoryAxisRenderer,
                ticks: ticks
            },
            // Pad the y axis just a little so bars can get close to, but
            // not touch, the grid boundaries.  1.2 is the default padding.
            yaxis: {
                pad: 1.05,
                tickOptions: {formatString: '$%d'}
            }
        }
    });
});
</script>
'''

class JQPlot(FORM):

    def __init__(self, query, 
                 row_field, col_field, data_field, 
                 #query=None,
                 row_ids=None, col_ids=None,
                 row_represent=None, col_represent=None, 
                 data_represent=None, data_parse=None,
                 row_totals=False, col_totals=False,
                 extra_values=None,
                 next=None,
                 **attributes):
        '''Muestra una hoja de calculo para editar una relacion que representa
        una funcion (ref1,ref2) -> valor
        '''
        FORM.__init__(self, **attributes)
        if not (row_field.table == col_field.table == data_field.table):
            raise AttributeError
        self.table = row_field.table
        
        if not row_ids:
            row_ids   = [r[row_field] for r in db(query).select(row_field, groupby=row_field)]
        row_names = [row_represent(rid) for rid in row_ids]
        if not col_ids:
            col_ids   = [r[col_field] for r in db(query).select(col_field, groupby=col_field)]
        col_names = [col_represent(cid) for cid in col_ids]
#TODO!!!

    def prepare_components(self):

        response.files.append(URL('static', 'jqplot/jquery.jqplot.min.js'))
        response.files.append(URL('static', 'jqplot/jquery.jqplot.min.css'))
        response.files.append(URL('static', 'jqplot/plugins/jqplot.barRenderer.min.js'))
        response.files.append(URL('static', 'jqplot/plugins/jqplot.categoryAxisRenderer.min.js'))
        response.files.append(URL('static', 'jqplot/plugins/jqplot.pointLabels.min.js'))

        partidas = db(db.partida.proyecto==PROYECTO).select(db.partida.id, db.partida.nombre, orderby=db.partida.id)
        jornadas = db(db.jornada.proyecto==PROYECTO).select(db.jornada.fecha, orderby=db.jornada.fecha)
        
        d = defaultdict(lambda :defaultdict(float))
        recs = db(query).select(
                    db.partida.id, db.jornada.fecha, data_field, 
                    groupby=db.partida.nombre|db.jornada.fecha
               )
        for row in recs:
            d[row.partida.id]['%s'%row.jornada.fecha] = row[data_field]
        data = [[float(d[row.id]['%s'%col.fecha]) for col in jornadas] for row in partidas]
        series_labels = ','.join("{label:'%s'}"%row.nombre for row in partidas)
        ticks = ['%s-%s'%(col.fecha.day, col.fecha.month) for col in jornadas]
        self.components = DIV(DIV(_id='chart1'), XML(js_jqplot%dict(data=data, series_labels=series_labels, ticks=ticks)))



