# -*- coding: utf-8 -*-
# (C) Copyright (C) 2012, 2013 Pablo Angulo
# This file is part of karakolas <karakolas.org>.

# karakolas is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as 
# published by the Free Software Foundation, either version 3 of the 
# License, or (at your option) any later version.

# karakolas is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU Affero General Public 
# License along with karakolas.  If not, see 
# <http://www.gnu.org/licenses/>.

from collections import defaultdict
import re
import gluon.sqlhtml
from gluon.storage import Storage
import random
from decimal import Decimal

################### TODO #############################
######################################################
#
# - Anayde a cada tabla un id, y asegurate de que puedan coexistir dos tablas distintas
# Este trabajo esta terminado en RTable, pero no he empezado en ROSheet ni en RWSheet
#
# - Permite seleccionar rangos, y luego poner el texto en el portapapeles, 
#adecuadamente tabulados. Este trabajo esta empezado en ROSheet, en el script js4rosheet
#
# - ERTable deberia permitir fijar las filas y el master field, de modo que sirva para
#editar los campos de las entradas, pero sin añadir ni quitar entradas. Luego refactoriza
#aceptar_productos_coordinados
#
# Busca otros TODO en el codigo
################### Widgets ##########################
######################################################

#Repito código de gluon.sqlhtml, pero cambiando el INPUT por un TEXTAREA
#Necesario para que funcione pegar rangos
#Pero por otro lado, se puede simplificar, porque sobreescribo el valor de class
#que diferencia por ej MyIntegerWidget de MyStringWidget

class DefaultStorage (Storage):
    def __init__(self, *args, **kwds):
        Storage.__init__(self, *args, **kwds)
    def __getitem__(self, item):
        try:
            return Storage.__getitem__(self, item)
        except:
            return None

class MyStringWidget(gluon.sqlhtml.FormWidget):
    _class = 'string'

    @classmethod
    def widget(cls, field, value, **attributes):
        """
        generates an INPUT text tag.

        see also: :meth:`FormWidget.widget`
        """

        default = dict(
            _type = 'text',
            value = (not value is None and str(value)) or '',
            _cols=20, _rows=1,             
            )
        attr = cls._attributes(field, default, **attributes)

        return TEXTAREA(**attr)


class MyIntegerWidget(MyStringWidget):
    _class = 'integer'


class MyDoubleWidget(MyStringWidget):
    _class = 'double'


class MyDecimalWidget(MyStringWidget):
    _class = 'decimal'


class MyTimeWidget(MyStringWidget):
    _class = 'time'


class MyDateWidget(MyStringWidget):
    _class = 'date'


class MyDatetimeWidget(MyStringWidget):
    _class = 'datetime'

##################### smart update #################
def _update_or_insert(table, keys, values):
    record = table(**keys)
    if record:
        record.update_record(**values)
        newid = record.id  #aqui diverjo de update_or_insert
    else:
        newid = table.insert(**values)
    return newid

def smart_update(table, query, rows, keys, 
                 action=lambda s:s.delete(),
                 params = None):
    '''If a row is already present in db(query), it is updated.
    If it is in rows but not in db(query), it is inserted.
    If a row is in db(query), but not in rows, it is deleted.
    
    rows is a list of dict's, as in table.insert(**row)
    '''
    #TO DO? Nunca deberia actualizar un registro que no esta en la query
    #UPDATE: No creo que se pueda resolver a este nivel
    if not isinstance(keys, (list, tuple)):
        keys = [keys]
    _db = table[table.fields[0]].db
    keys2id = {}
    for row in rows:
        keys0  = tuple(row[key] for key in keys)
        keys0d = dict(zip(keys, keys0))
        if params: keys0d.update(params)
        id = _update_or_insert(table, keys0d, row)
        keys2id[keys0] = id
    
    ids_query = set(row.id for row in _db(query).select(table.id))
    removed   = ids_query.difference(keys2id.values())
    if removed:
        action( _db(query & table.id.belongs(removed)) )
    return keys2id

#################### ERTable #######################
####################################################

#Los ids de estos controles no llevan id porque nunca deberia haber mas de uno.
ertable_controls = DIV(*[INPUT(_type='submit', _name='save_rtable_data',    _value=T('Guardar cambios')), 
                       INPUT(_type='submit', _name='discard_rtable_data', _value=T('Descartar cambios'))],
                      **dict(_class='rtable_controls'))

ert_script_add_rows = '''<script type="text/javascript">
add_rows_%(id)s = function(rows){
    var table  = $('#ertable_%(id)s').find('tbody');
    var ufila  = $('#_ultima_fila_%(id)s').val();
    var fila0  = $('#_sample_row_%(id)s').val();
    for(var i=0;i<rows;i++){
        var fila = fila0.replace(/(cell_[a-z_0-9]*_)0/g, '$1'+ufila).replace('ertable_%(id)s_row_0','ertable_%(id)s_row_'+ufila);
        table.append(fila);
        ufila++;
    }
    $('#_ultima_fila_%(id)s').val(ufila);
    prepare_paste_behaviour_%(id)s();
    prepare_navigation_%(id)s();
    jQuery('[multiple]').multiSelect(); //in case there are multiple selection widgets
    $('#ertable_%(id)s').trigger('nueva_fila');
}
</script>
'''

ert_script_add_columns = '''<script type="text/javascript">
//TODO: replacements = {' ':'_', 'á':'a', 'é:'e', 'í':'i', 'ó':'o', 'ú':'u', 'ñ','n'};
delete_col_%(id)s = function(col_name){
    $('*[id^="ertable_%(id)s_cell_' + col_name + '"]').parent().detach();
    $('#ertable_%(id)s_th_'+col_name).detach();
}

nueva_columna_%(id)s = function(){
    var trs = $('#ertable_%(id)s tr');
    var nombre = $('#ertable_%(id)s_nombre_nueva_columna').val();
    //TODO: problema: ya uso la barra baja para los ids de las celdas
    //este doble uso e un problemon
    var nombre_n = nombre.replace(' ','_').toLowerCase();
    var tipo_largo = $('#ertable_%(id)s_tipo_nueva_columna').val();
    var tipo;
    if (tipo_largo.substring(0,7)=='decimal'){
        tipo='decimal';
    }else{
        tipo=tipo_largo;
    }

    if(!nombre_n){
        alert('Por favor escribe un nombre de columna y un tipo de datos');
        return true;
    }
    var fields = $('#_ertable_%(id)s_fields').val();
    var field_array = fields.split(',');
    for(var k=0;k<field_array.length;k++){
        if(nombre_n===field_array[k]){
            alert('Este nombre de columna ya está en uso');
            return true;
        }
    }
    var sample_td = $('#_ertable_%(id)s_sample_td_'+tipo).val();
    var my_td = sample_td.replace(/new_field_name_0/g,nombre_n+'_XXX');
    
    var sample_th = $('#_ertable_%(id)s_sample_th').val();
    trs[0].innerHTML += sample_th.replace(/new_field_name/g,nombre_n).replace(/new_field_type/g,tipo_largo);
    for(var j=1; j<trs.length; j++){
        var rid = trs[j].id.split('_')[3];
//no puedo hacer trs[j].innerHTML += my_td.replace(/XXX/g,rid);
//porque innerHTML no se actualiza al introducir valores en los campos
//asi que los campos vuelven a sus valores por defecto
        $('#ertable_%(id)s tbody tr:nth-child('+j+')').append(my_td.replace(/XXX/g,rid) );


    }
    var fila0  = $('#_sample_row_%(id)s').val();
    var new_sample_row = fila0.slice(0,-5) + sample_td.replace(/new_field_name_0/g,nombre_n+'_0'); +'</tr>';    
    $('#_sample_row_%(id)s').val(new_sample_row);

    $('#_ertable_%(id)s_fields').val(fields + ',' + nombre_n);
    prepare_paste_behaviour_%(id)s();
    prepare_navigation_%(id)s();
    $('.delete_col_button').show();
    $('#ertable_%(id)s').trigger('nueva_columna');
}
</script>
'''

ert_script_paste_rtable = r'''<script type="text/javascript">
strippable = '" \t\n' + "'";
strip = function(s){
    var j, startj=0, endj=s.length-1;
    for(j=0; j<s.length;j++){
        if(strippable.indexOf(s[j])<0){
            startj=j;
            break;
        }
    }
    for(j=s.length-1;j>=0;j--){
        if(strippable.indexOf(s[j])<0){
            endj=j;
            return s.substring(startj, endj+1);
        }
    }

}

text2array = function(text){
            if (!text.match(/[\r\n\t]/)){
                return [[text]];
            }

            //Eliminar retornos de linea entrecomillados
            //Se puede mejorar
            var lastj = 0;
            var in_commas = false;
            var ftext ='';
            for(var j = 0; j<text.length; j++){
                if(text[j]=='"'){
                    if(in_commas){
                        ftext += text.substring(lastj, j).replace(/(\r\n|\n|\r)/gm,'');
                    }
                    else{
                        ftext += text.substring(lastj, j);
                    }
                    in_commas = !in_commas;
                    lastj=j;
                }
            }
            ftext += text.substring(lastj);
            text=ftext;

            var filas  = (text+'\n').split(/\r\n|\r|\n/);
            var result = [];
            var i = 0;
            for(var i=0;i<filas.length;i++){
                var fila   = filas[i];
                if(!fila.trim()){
                    continue;
                }
//                    var celdas = fila.split(/[\t,]/);
                var celdas = [];
                if(fila.match(/\t/)){
                    var preceldas = fila.split(/\t/);
                    for (var g=0;g<preceldas.length;g++){
                        celdas.push(strip(preceldas[g]))
                    }
                    
                }else if(fila.match(/"/)){  
                    //Vaya mierda: no se si se puede a golpe de regex, asi que parseo.
                    fila = fila+',';
                    var f1 = -1;
                    var f2 = -1;
                    var last = 0;
                    for(var f=0;f<fila.length;f++){
                    if(fila[f]=='"'){
                        if(f1<0){
                            f1=f;
                        }
                        else if(f2<0){
                            f2=f;
                        }
                        else{
                            continue;//ERROR
                        }
                    }
                    else if(fila[f]===','){
                        if(f1<0 && f2<0){
                            celdas.push(strip(fila.substring(last,f)));
                            last=f+1;
                        }
                        else if(f1>=0 && f2>0){
                            celdas.push(strip(fila.substring(last+1,f-1)));
                            last=f+1;
                            f1=-1;
                            f2=-1;
                        }
                      }
                    }
                }else{
                    celdas = fila.split(/,/);
                }
                result.push(celdas);
            }
            
        return result;
}

prepare_paste_behaviour_%(id)s = function() {
    $("#ertable_%(id)s textarea").bind('paste', function(e) {
        var n = this.name;
        $('#'+n).val('');
        $('#_ertable_%(id)s_paste_into_cell').val(n);
        // Short pause to wait for paste to complete
        setTimeout( function() {
            var target = $('#_ertable_%(id)s_paste_into_cell').val();
            var text   = $('#'+target).val();
            var fields  = $('#_ertable_%(id)s_fields').val().split(',');
            var ls = target.split('_');
                        
            var pfila = parseInt(ls[ls.length-1]);
            var ufila  = $('#_ultima_fila_%(id)s').val();
            
            var pcol = ls.slice(3,-1).join('_');
            var pcolu = 0;
            while((pcolu<fields.length) & (fields[pcolu]!=pcol)){
                pcolu++;
            }
            var ucolu = parseInt(fields.length);

            var lines = text2array(text);
            
            var ifila  = pfila;
            for(var i = 0; i<lines.length; i++){
                var celdas = lines[i];
                var j = 0;
                var icolu = pcolu;
                while((icolu<ucolu) && (j<celdas.length)){
                    var ctag  = $('#ertable_%(id)s_cell_'+fields[icolu]+'_'+ifila);
                    var celda = celdas[j]?celdas[j].trim():'';
                    //http://stackoverflow.com/questions/149573/check-if-option-is-selected-with-jquery-if-not-select-a-default
                    if(ctag[0].tagName=='SELECT'){
                        $('#ertable_%(id)s_cell_'+fields[icolu]+'_'+ifila+' option:contains("'+celda+'")').attr('selected', 'selected');
                    }else if(ctag[0].type=='checkbox'){
                        if(celda){
                            $('#ertable_%(id)s_cell_'+fields[icolu]+'_'+ifila).attr('checked', 'checked');
                        }
                    }else{
                        $('#ertable_%(id)s_cell_'+fields[icolu]+'_'+ifila).val(celda);
                    }
                    j++;
                    icolu++;
                }
                var next_row = $('#ertable_%(id)s_row_'+ifila).next()[0];
                if(!next_row){
                    break;
                } 
                ifila = next_row.id.split('_')[3];
            }
        }, 100);
    });
}
$(document).ready(prepare_paste_behaviour_%(id)s);
</script>
'''

#TODO: Al cambiar de tamaño un textarea de un RTable, se cambia toda la fila o columna (usa el evento resize: http://api.jquery.com/resize/)
#Despues recicla para un RWSheet, si puedes (fixheadertable sera un estorbo, o quiza una inspiracion, por su uso de colgroup y col)
ert_script_resize_rtable = r'''

'''

#Keyboard navigation
ert_script_navigation = r'''<script type="text/javascript">
prepare_navigation_%(id)s = function(){
    $("#ertable_%(id)s textarea, #ertable_%(id)s input").bind('keydown', function(e) {
        var next_row, next_cell, next_row_id, fieldname, myrow;
        if(e.keyCode==13 || e.keyCode==40 || e.keyCode==38){//enter, down arrow or up arrow
            try{
                fieldname = this.id.split('_').slice(3,-1).join('_');
                myrow = $(this).parent().parent();
                if(e.keyCode==13 || e.keyCode==40){
                    next_row = myrow.next();
                }else if(e.keyCode==38){
                    next_row = myrow.prev();
                }
                next_row_id = next_row[0].id.split('_')[3]
                next_cell = next_row.find('#ertable_%(id)s_cell_' + fieldname + '_' + next_row_id);
                next_cell.focus();
                next_cell.select();
            }catch(_){}
            e.preventDefault();     
        }else if(e.keyCode==39 || e.keyCode==37){//right or left arrow
            //Forget this, TAB is good enough, arrows are needed to move within the textarea and this can be confusing
        }else if(e.keyCode==9){
            setTimeout(function(){
                $( document.activeElement ).select();
            },100);
        }
    });
$("#ertable_%(id)s input[type='checkbox']").bind('click',function(e){
    this.focus();
});
}
$(document).ready(prepare_navigation_%(id)s);
</script>
'''

#Fix master field
#TODO: Can't put two ERTable in the same page with different
#fix_master_field setting
#TODO: Busca el equivalente a readonly=readonly para select/option en html
ert_fix_master_fields = r'''<script type="text/javascript">
$(document).ready(function(){
    $(".ertable_master_field").attr('readonly', 'readonly');
});
</script>
'''

class ERTable(FORM):
    #TODO: custom widget?
    widgets = Storage(dict(
        string = MyStringWidget,
        text = gluon.sqlhtml.TextWidget,
        integer = MyIntegerWidget,
        double = MyDoubleWidget,
        decimal = MyDecimalWidget,
        time = MyTimeWidget,
        date = MyDateWidget,
        datetime = MyDatetimeWidget,
        boolean = gluon.sqlhtml.BooleanWidget,
        options = gluon.sqlhtml.OptionsWidget,
        multiple = gluon.sqlhtml.MultipleOptionsWidget,
        password=gluon.sqlhtml.PasswordWidget,
#        radio = gluon.sqlhtml.RadioWidget,
#        checkboxes = gluon.sqlhtml.CheckboxesWidget,
#        autocomplete = gluon.sqlhtml.AutocompleteWidget,
#        list = gluon.sqlhtml.ListWidget,
        ))    

    def __init__(self, table,
                 fields=None,
                 query=None,
                 orderby=None,
                 extra_columns_table=None,
                 extra_columns_values=None,
                 extra_columns_names=None,
                 extra_data=None,
                 extra_data_fields=None,
                 url='.',
                 next=None,
#                 parsers=None,
                 values=None,
                 add_controls=True,
                 add_new_columns=False,
                 master_fields=None,
                 fix_master_fields=False,
                 add_n_rows_at_a_time=(5,25),
                 **attributes):
        '''ERTable builds a spreadsheet-like view that allows to edit a whole table,
        or a slice of it, at once
        '''
        FORM.__init__(self, **attributes)
        self.table  = table
        self.db     = table['id'].db
        self.url    = url
        self.values = values or {}
        self.fields = fields or [f.name for f in table if 
                (f.type!='id' and (not f.compute) and f.name not in self.values) ]
        if query:
            self.query = query
        else:
            if values:
                self.query = reduce(lambda a,b:(a & b) , 
                                    [table[t]==v 
                                     for t,v in self.values.iteritems() ])
            else:
                self.query = (table.id>0)
        self._id     = attributes.get('_id','') or str(random.random())[2:]
        self._class  = attributes.get('_class','ertable')
        if self._class != 'ertable': self._class+=',ertable'
        self.next    = next
        self.add_controls = add_controls
        self.add_new_columns = add_new_columns
        self.add_n_rows_at_a_time = add_n_rows_at_a_time
        self.fix_master_fields = fix_master_fields and master_fields
        if not master_fields:
            if hasattr(self.table,'_keys'):
                master_fields = self.table._keys
            else:
                try:
                    master_fields = [(f.name for f in self.table if f.unique).next()]
                except StopIteration:
                    master_fields = self.fields
        self.master_fields = master_fields
        if orderby:
            self.orderby = orderby
        else:
            self.orderby =  reduce(lambda a,b:(a | b) , 
                                   (self.table[f] for f in self.master_fields))
#        raise Exception, (str(self.orderby), self.master_fields)
        if (not (extra_columns_table and extra_columns_values and 
                  extra_data and extra_data_fields) 
            or self.fix_master_fields):
            self.has_extra = self.add_new_columns = False
        else:
            self.has_extra = True
            self.extra_columns_table  = extra_columns_table
            self.extra_columns_values = extra_columns_values
            self.extra_columns_name_field = self.extra_columns_type_field = None

            #Lo que sigue no rula porque no veo una buena forma de distinguir
            #el campo "nombre" del campo "tipo"
#            for f in self.extra_columns_table.fields:
#                t = self.extra_columns_table[f]
#                if f in self.extra_columns_values or f=='id':
#                    pass
#                elif t.type=='string' and not self.extra_columns_name_field:
#                    self.extra_columns_name_field = f
#                elif t.type=='string' and not self.extra_columns_type_field:
#                    self.extra_columns_type_field = f
#                else:
#                    raise Exception, "extra_columns_table must be a set of tuples "\
#                                      "(column_name, column_type)"
            (self.extra_columns_name_field,
             self.extra_columns_type_field) = extra_columns_names

            self.extra_data           = extra_data
            (self.extra_data_row_field, 
             self.extra_data_column_field,
             self.extra_data_value_field) = extra_data_fields
            #We might check here that extra_data is consistent

#        if not parsers:
#            parsers = [(lambda x:x or table[f].default) for f in fields]
#       self.parsers = parsers
        self.last_submission = []
        
        self.prepare_components()

    def prepare_extra(self):
        self.query_extra = reduce(lambda a,b:(a & b) , 
                                  [self.extra_columns_table[t]==v 
                                   for t,v in self.extra_columns_values.iteritems() ])
        s = db(self.query_extra).select()
        self.extra_columns = [(row.id,
                               row[self.extra_columns_name_field],
                               row[self.extra_columns_type_field]) for row in s]

    def widget_of_type(self, field_type):
        if field_type.startswith('decimal'):
            field_type = 'decimal'
        if field_type.startswith('reference'):
            field_type = 'options'
        if field_type.startswith('list:reference'):
            field_type = 'multiple'
        try:
            return self.widgets[field_type].widget
        except AttributeError:
            raise AttributeError('ERTable not implemented for fields of type %s'%field_type)
        
    def field_widget(self, f):
        field = self.table[f]
        if gluon.sqlhtml.OptionsWidget.has_options(field):
            if  hasattr(field.requires, 'multiple') and field.requires.multiple:
                return gluon.sqlhtml.MultipleOptionsWidget.widget
            return gluon.sqlhtml.OptionsWidget.widget
        field_type = self.table[f].type
        return self.widget_of_type(field_type)

    def new_td(self, widget, content, field, fieldname, typename=None, row_number=-1):
        row_number = row_number if row_number>=0 else self.row_count
        class_type = (self.table[fieldname].type.replace(' ','_') 
                        if hasattr(self.table, fieldname)
                        else typename)
        return TD(
           widget(field, content, 
                  _class='ertable_field_%s %s%s'%(
                     typename or fieldname, class_type,
                     ' ertable_master_field' if fieldname in self.master_fields else '' ),
                  _id='ertable_%s_cell_%s_%d'%(self._id, fieldname, row_number), 
                 _name='ertable_%s_cell_%s_%d'%(self._id,  fieldname, row_number))
        )

    def new_tr(self, row=None, row_extra=None, increase_counter=1):
        row = row or dict((f,self.table[f].default) for f in self.fields)
        row_extra = row_extra or defaultdict(str)

        tds = [self.new_td(w, row[f], self.table[f], f)
                for f,w in zip(self.fields, self.field_widgets)]
        if self.has_extra:
            tds += [self.new_td(w, row_extra[f], 
                                self.extra_data[t][self.extra_data_value_field], 
                                f, t.split('(')[0])
                    for (_,f,t),w in zip(self.extra_columns,
                                    self.extra_field_widgets)]

        mytr = TR(TD(A(IMG(_src=URL('static','images/delete.png'), 
                           _width="16", _class='delete_row_button'), 
                       callback=URL(c='default', f='none'), delete='tr' )),
                  *tds,
                  **dict(_id='ertable_%s_row_%d'%(self._id, self.row_count)))
        self.row_count+=increase_counter
        return mytr

    def new_extra_th(self, field, type):
        return TH(INPUT(_type='hidden', 
                         _name='ertable_%s_nueva_columna_%s'%(self._id, field),
                         _value=type),
                   field, XML('&nbsp;'),
                   SPAN(XML('&nbsp;x&nbsp;'), 
                       _onclick='delete_col_%s("%s")'%(self._id, field),
                       _class='delete_col_button'),
                   XML('&nbsp;'),
                   _class='extra_th',
                   _id='ertable_%s_th_%s'%(self._id,field))

    def _traverse(self, status, hideerror=False):
        newstatus = status
        for c in self.elements('tr'):
            try:
                row_no = int(c.attributes['_id'].split('_')[-1])
            except (ValueError, KeyError) as e:
                continue
            if row_no not in self.good_rows:
                continue
#        for c in self.components:
            if hasattr(c, '_traverse') and callable(c._traverse):
                c.vars = self.vars
                c.request_vars = self.request_vars
                c.errors = self.errors
                c.latest = self.latest
                c.session = self.session
                c.formname = self.formname
                if hideerror and not \
                        self.attributes.get('hideerror', False):
                    c['hideerror'] = hideerror
                newstatus = c._traverse(status, hideerror) and newstatus

        # for input, textarea, select, option
        # deal with 'value' and 'validation'

        name = self['_name']
        if newstatus:
            newstatus = self._validate()
            self._postprocessing()
        elif 'old_value' in self.attributes:
            self['value'] = self['old_value']
            self._postprocessing()
        elif name and name in self.vars:
            self['value'] = self.vars[name]
            self._postprocessing()
        if name:
            self.latest[name] = self['value']
        return newstatus

    def prepare_components(self):
        if self.has_extra:
            self.prepare_extra()

        self.row_count = 0
#        cols = COLGROUP(*[COL(_id='ertable_%s_row_%s'%(self._id, f)) 
#                            for f in self.fields] + 
#                         [COL(_id='ertable_%s_row_%s'%(self._id, f)) 
#                            for _,f,_ in self.extra_columns])

        ths = [TH(f, _id='ertable_%s_th_%s'%(self._id,f)) 
                                for f in self.fields]
        if self.has_extra:
            ths += [self.new_extra_th(f,t) 
                    for _,f,t in self.extra_columns]
        headers = THEAD(TR(TH(), *ths))
        
        self.field_widgets = [self.field_widget(f) for f in self.fields]        
        std_rows = self.db(self.query).select(self.table.id, 
                                              *[self.table[f] for f in self.fields],
                                              **dict(orderby=self.orderby))

        vs_extra = dict((row.id,defaultdict(str)) for row in std_rows)
        if self.has_extra:
            self.extra_field_widgets = [self.widget_of_type(t) for _,_,t in self.extra_columns]
            for cid,f,t in self.extra_columns:
                val_table = self.extra_data[t] 
                s = db((val_table[self.extra_data_column_field]==cid) &
                       self.query)\
                    .select(val_table[self.extra_data_value_field],
                            val_table[self.extra_data_row_field])
                for row in s:
                    #The KeyError can happen if some extra data refers to a row
                    #that is not part of self.query
                    try:
                        vs_extra[row[self.extra_data_row_field]][f]=row[self.extra_data_value_field]
                    except KeyError:
                        continue

        sample_row = self.new_tr(increase_counter=0).xml()
        rows     = [self.new_tr(row, row_extra=vs_extra[row.id]) 
                    for row in std_rows]
        if self.fix_master_fields:
            new_rows = []
        else:
            new_rows = [self.new_tr() for _ in range(min(self.add_n_rows_at_a_time))]

        #TODO: incluye colgroup y cols (ver mas arriba un boceto)
        table = TABLE(headers, *(rows+new_rows), 
                      **dict(_class=self._class, _id='ertable_%s'%self._id))
        add_rows_buttons = [DIV(T('Añadir %d filas más'%k), 
                                  _onclick="add_rows_%s(%d);"%(self._id, k),
                                  _class='boton_anade_%d_filas'%k)
                               for k in self.add_n_rows_at_a_time]
        c = [INPUT(_type='hidden', 
                   _id='_ertable_%s_fields'%self._id, 
                   _name='_ertable_%s_fields'%self._id,
                   _value=','.join(self.fields +
                                   [f for _,f,_ in self.extra_columns] 
                                   if self.has_extra else self.fields)),
             INPUT(_type='hidden', 
                   _id='_ultima_fila_%s'%self._id,
                   _name='_ultima_fila_%s'%self._id, 
                   _value=self.row_count),
             INPUT(_type='hidden', 
                   _id='_table_name_%s'%self._id,
                   _name='_table_name_%s'%self._id,
                   _value=str(self.table)),
             INPUT(_type='hidden', 
                   _id='_ertable_%s_paste_into_cell'%self._id,
                   _name='_ertable_%s_paste_into_cell'%self._id, 
                   _value='nombre_0'),
             INPUT(_type='hidden', 
                   _id='_sample_row_%s'%self._id,
                   _name='_sample_row_%s'%self._id, 
                   _value=sample_row)]
        hidden_fields   = DIV(c, _class="hidden")
        js_paste        = XML(ert_script_paste_rtable%dict(id=self._id))
        js_add_rows     = XML(ert_script_add_rows%dict(id=self._id))
        js_navigation   = XML(ert_script_navigation%dict(id=self._id))
        js_fix_mf       = XML(ert_fix_master_fields)
        if self.fix_master_fields:
            self.components = [table, hidden_fields, js_paste, js_navigation, js_fix_mf]
        else:
            self.components = ([table] +
                               add_rows_buttons +
                               [hidden_fields, js_add_rows, js_paste, js_navigation]
                               )
        if self.add_new_columns:
            extra_tds = [(self.new_td(self.widget_of_type(t), 
                                      '', 
                                      self.extra_data[t][self.extra_data_value_field], 
                                      'new_field_name', 
                                      t.split('(')[0],
                                      row_number = 0 ) ,
                          t.split('(')[0])
                          for t in TABLAS_VALORES_EXTRA]
            sample_tds = [INPUT(_type='hidden', 
                                _id='_ertable_%s_sample_td_%s'%(self._id, t),
                                _name='_ertable_%s_sample_td_%s'%(self._id, t), 
                                _value=td.xml())
                           for td,t in extra_tds]
            sample_th = INPUT(_type='hidden',
                              _id='_ertable_%s_sample_th'%self._id,
                              _name='_ertable_%s_sample_th'%self._id, 
                              _value=self.new_extra_th('new_field_name','new_field_type').xml())
            hidden_data_new_columns = DIV(sample_th, *sample_tds, **dict(_class="hidden"))

            js_add_columns = XML(ert_script_add_columns%dict(id=self._id))

            nombre_nueva_columna = INPUT(_id='ertable_%s_nombre_nueva_columna'%self._id)
            tipo_nueva_columna   = SELECT(*[OPTION(nombre_tipo, _value=tipo) 
                                          for (tipo, nombre_tipo) in 
                                           sorted(NOMBRES_TIPOS_COLUMNA.iteritems())],
                                  **dict(_id='ertable_%s_tipo_nueva_columna'%self._id))

            boton_nueva_columna  = DIV('Nueva columna', _class='boton_nueva_columna',
                                      _onclick='nueva_columna_%s();'%self._id)
            div_nueva_columna = DIV(FIELDSET(nombre_nueva_columna,
                                             tipo_nueva_columna,
                                             boton_nueva_columna,
                                             hidden_data_new_columns,
                                             js_add_columns),
                                    _id='ertable_%s_div_nueva_columna'%self._id,
                                    _class="nueva_columna")

            self.components.insert(0, div_nueva_columna)

        if self.add_controls:
            self.components.append(ertable_controls)
    response.files.append(URL('static', 'css/rsheet.css'))
    response.headers['X-XSS-Protection'] = '0'


    #TODO: Piensa como evitar que esta funcion este aqui.
    #Quiza hay que pasar un argumento extra al constructor
    def parser(self, type, default=None):
        if type.startswith('decimal'):
            return lambda v:Decimal(v.replace(',','.') if (v and v!='None') else '0')
        return lambda v:v # or default

    def field_parser(self, f):
        return self.parser( self.table[f].type , self.table[f].default)

    def accepts(
        self,
        request_vars,
        session=None,
        formname=None,
        keepvalues=False,
        onvalidation=None,
        dbio=True,
        hideerror=False,
#        detect_record_change=False,
        on_delete=lambda s:s.delete()
        ):
        if request_vars.__class__.__name__ == 'Request':
            request_vars=request_vars.post_vars
        request_vars = DefaultStorage(request_vars)
        #if they come here by a hit on "discard changes", don't save changes
        if self.add_controls:
            if request_vars.discard_rtable_data:
                if self.next:
                    redirect(self.next)
                else:
                    return False
            inmersed = request_vars.inmmersed_form
            saved    = request_vars.save_rtable_data
            if (not saved) and (not inmersed):
                return False

        self.errors.clear()
        self.request_vars = Storage()
        self.request_vars.update(request_vars)
        self.session = session
        self.formname = formname
        self.keepvalues = keepvalues
        
        ufila = int(request_vars['_ultima_fila_%s'%self._id])
        self.good_rows = {}
        repeated_row_errors = {}
        
        keys = set()
        for i in xrange(ufila):
            mvs = []
            for mf in self.master_fields:
                k  = 'ertable_%s_cell_%s_%d'%(self._id, mf, i)
                mv = request_vars[k]
                mvs.append(mv)
            if mvs and any(mv and mv.strip() for mv in mvs):
                key = tuple((mv.strip() if mv else '') 
                            for mv in mvs)
                if key in keys:
                    for mf in self.master_fields:
                        k  = 'ertable_%s_cell_%s_%d'%(self._id, mf, i)
                        if len(self.master_fields)==1:
                            repeated_row_errors[k]=T('The value for this field was used in a different row')
#                            self.element('#%s'%k)
                        else:
                            repeated_row_errors[k]=T('This combination of %s was used in a different row')%self.master_fields
                keys.add(key)
                self.good_rows[i] = key

        #1:llama al metodo de FORM para comprobar que cada campo sea correcto
        #Problema: no es tolerante con las filas vacías
        #Asi que leo self.errors y si no hay errores en las good_rows,
        #lo doy por bueno, y reseteo self.errors
#        ret = self._traverse(True, hideerror)

        ret = FORM.accepts(
            self,
            request_vars,
            session,
            formname,
            keepvalues,
            onvalidation,
            hideerror=hideerror,
            )

        def error_in_good_row(k):
            try:
                return int(k.split('_')[-1]) in self.good_rows 
            except:
                return False

        to_pop = set()
        for k,v in self.errors.iteritems():
            if (not error_in_good_row(k)) or (v=='value already in database or empty'):
                to_pop.add(k)
        for k in to_pop:
            self.errors.pop(k)
#        self.errors = Storage((k,e) for (k,e) in self.errors.iteritems()
#                               if error_in_good_row(k))
        self.errors.update(repeated_row_errors)

#        def not_in_good_row(k):
#            try:
#                return int(k.split('_')[-1]) not in self.good_rows 
#            except:
#                return True
        ret = ret or (not self.errors)
#        ret = ret or (#self.table[self.master_field].unique and
#                      all(e=='value already in database or empty' 
#                         for e in self.errors.values()))
        self.last_submission = []
        if ret and dbio:   
            self.errors.clear()         
            parsers = [self.field_parser(f) for f in self.fields]

            rows = []
            for i in self.good_rows:
                vs = dict((f,p(request_vars['ertable_%s_cell_%s_%d'%(self._id,f,i)])) 
                          for f,p in zip(self.fields, parsers))
                vs.update(self.values)
                rows.append(vs)
            
            if self.master_fields:
                mf2id = smart_update(self.table,
                                     self.query,
                                     rows,
                                     self.master_fields,
                                     action=on_delete,
                                     params=self.values)
            else:
                raise NotImplementedError, "For this table, you need to specify the master fields explicitely"
                #update a pelo
                db(self.query).delete()
                #TODO: Insert
                #HERE
                
            #Columnas extra
            if self.has_extra:
                nuevas_columnas = [(v.split('_')[-1],t) for (v,t) in request_vars.iteritems() 
                                  if v.startswith('ertable_%s_nueva_columna_'%self._id)]
                rows = []
                for n,t in nuevas_columnas:
                    vs = {self.extra_columns_name_field:n, 
                          self.extra_columns_type_field:t}
                    vs.update(self.extra_columns_values)
                    rows.append(vs)
                extra_nf2id = smart_update(self.extra_columns_table,
                                           self.query_extra,
                                           rows,
                                           self.extra_columns_name_field)
                
                for f,t in nuevas_columnas:
                    cid = extra_nf2id[(f,)]
                    #TODO: use smart_update with compound primary key
                    self.db( self.extra_data[t][self.extra_data_column_field]==cid).delete()
                    for i,mf in self.good_rows.iteritems():
                        v = self.parser(t) (request_vars['ertable_%s_cell_%s_%d'%(self._id,f,i)])
                        vs = {self.extra_data_row_field:mf2id[mf],
                              self.extra_data_column_field:cid,
                              self.extra_data_value_field:v}
                        #TODO: use smart_update with compound primary key
                        self.extra_data[t].insert(**vs)                
        
        return ret

    def process_request(self, request_vars, keep_order=True):
        '''call after accepts returns true
        '''
        rows = {}
        for i,t in self.good_rows.iteritems():
            rows[t] = dict((f,request_vars['ertable_%s_cell_%s_%d'%(self._id, f, i) ] )
                           for f in self.fields)
            if keep_order:
                rows[t]['order'] = i
        return rows

    @staticmethod
    def factory(*fields, **attributes):
        """
        generates a ERTable for the given fields.

        Internally will build a non-database based data model
        to hold the fields.
        """
        # Define a table name, this way it can be logical to our CSS.
        # And if you switch from using ERTable to ERTable.factory
        # your same css definitions will still apply.

        table_name = attributes.get('table_name', 'no_table')

        # So it won't interfear with SQLDB.define_table
        if 'table_name' in attributes:
            del attributes['table_name']

        return ERTable(DAL(None).define_table(table_name, *fields),
                       **attributes)


######################### ROSheet ############################
##############################################################

#Sigue un ejemplo de como podria conseguirse copiar y pegar rangos.
#Por ahora solo he escrito codigo para que con un click se seleccione una celda
#y con mayus+click se extienda a un rango, y ademas solo funciona en chromium.
#Habria que conseguir que en vez de cambiar el color de fondo cambiara la clase
#css de la celda. Usando la documentacion de jquery, las herramientas para 
#desarrolladores de chromium y el firebug ser facil.
#tb hace falta anular el comportamiento por defecto del mayusculas+click
#Para el ROSheet solo tiene sentido copiar rangos. Faltaria hacer bind de 
#ctrl+c a una funcion que devuelva el rango seleccionado, las filas separadas por
#retornos de linea, las columnas por tabuladores.
js4rosheet  = XML(r'''<script type="text/javascript">
prepare_cells = function() {
    $(".rosheet td").bind('click', function(e) {
        var ns, row, col, crow, ccol, r, c, lrow, lcol;
            if(e.shiftKey){
                if($('#selected_cell').html()){
                    ns  = $('#selected_cell').html().substring(5).split('_');
                    row = ns[0]; col = ns[1];
                    ns  = e.srcElement.id.substring(5).split('_');
                    crow = ns[0]; ccol = ns[1];
                    ns  = $('#shift_selected_cell').html().substring(5).split('_');
                    lrow = ns[0]; lcol = ns[1];
                    for(r=row;r<=lrow;r++){
                        for(c=col;c<=lcol;c++){
                            $('#cell_'+r+'_'+c)[0].style.backgroundColor='#FFF';
                        }
                    }
                    if(crow<row || ccol<col){
                        $('#shift_selected_cell').html('');
                    }else{
                        $('#shift_selected_cell').html(e.srcElement.id);
                        for(r=row;r<=crow;r++){
                            for(c=col;c<=ccol;c++){
                                $('#cell_'+r+'_'+c)[0].style.backgroundColor='#FCC';
                            }
                        }
                    }
                }else{
                    $('#shift_selected_cell').html('');
                    $('#selected_cell').html(e.srcElement.id);
                }
            }else{
                if($('#selected_cell').html() && $('#shift_selected_cell').html() ){
                    ns  = $('#selected_cell').html().substring(5).split('_');
                    row = ns[0]; col = ns[1];
                    ns  = $('#shift_selected_cell').html().substring(5).split('_');
                    lrow = ns[0]; lcol = ns[1];
                    for(r=row;r<=lrow;r++){
                        for(c=col;c<=lcol;c++){
                            $('#cell_'+r+'_'+c)[0].style.backgroundColor='#FFF';
                        }
                    }
                }
                $('#shift_selected_cell').html('');
                if($('#selected_cell').html()){
                    $('#'+$('#selected_cell').html())[0].style.backgroundColor='#FFF';
                }
                $('#selected_cell').html(e.srcElement.id);
                $('#'+e.srcElement.id)[0].style.backgroundColor='#FCC';
            }
        e.preventDefault();     
        }
    );
}
$(document).ready(prepare_cells);
</script>
''')

selected      = SPAN('', _id='selected_cell', _style='visibility:hidden')
shift_selected = SPAN('', _id='shift_selected_cell', _style='visibility:hidden')

def ROSheet(query, row_field, col_field, data_field, 
            row_ids=None, col_ids=None,
            row_represent=None, col_represent=None, data_represent=None,
            _id=''):
    '''Representacion de una query indexada por un campo fila y otro columna
    
    row_ids es una lista de ids de registros a usar en las filas, si el campo
        row_field es de tipo reference
    lo mismo para col_ids
    '''
    if not row_represent:
        row_represent = row_field.represent or (lambda x:x)
    if not col_represent:
        col_represent = col_field.represent or (lambda x:x)
    if not data_represent:
        try:
            data_represent = data_field.represent or (lambda x:x)
        except AttributeError:
            #A f.sum() or f.count() doesn't have a represent attribute
            data_represent = (lambda x:x)

    #En ROSheet, row_ids no es necesariamente una lista de enteros
    if not row_ids:
        row_ids   = [r[row_field] for r in db(query).select(row_field, groupby=row_field)]
    row_names = [row_represent(rid) for rid in row_ids]
    if not col_ids:
        col_ids   = [r[col_field] for r in db(query).select(col_field, groupby=col_field)]
    col_names = [col_represent(cid) for cid in col_ids]

    table_data = defaultdict(str)
    for q_row in db(query).select(row_field, col_field, data_field, groupby=row_field | col_field):
        r,c,d = q_row[row_field], q_row[col_field], q_row[data_field]
        table_data[(r,c)] = data_represent(d) or ''
    col_headers = THEAD(TR(TH(''),*[TH(n) for n in col_names]))
    trows = [TR(TH(row_represent(r)), 
               *[TD(table_data[(r, c)], _id='cell_%d_%d'%(j,k)) for k,c in enumerate(col_ids)]) 
            for j,r in enumerate(row_ids) ]
    response.files.append(URL('static', 'css/rsheet.css'))
    return DIV(#css4rosheet, #js4rosheet, selected, shift_selected,
                TABLE(col_headers, *trows, **{'_class':'rosheet', '_id':_id}))

######################### RWSheet ############################
##############################################################

rwsheet_controls = DIV(*[INPUT(_type='submit', _name='save',    _value=T('Guardar cambios')), 
                INPUT(_type='submit', _name='discard', _value=T('Descartar cambios'))],
              **dict(_class='rwsheet_controls'))

show_product_area = DIV(P(''),_id='active_product')

class RWSheet(FORM):
    #TODO: Podria ser util tb un conjunto de booleanos?
    #TODO: custom widget? widget no uniforme?
    #Cualquiera de estas dos podria ser util para poder usar un decimal si es
    #a granel o un entero si no lo es.
    widgets = Storage(dict(
        string = MyStringWidget,
        integer = MyIntegerWidget,
        double = MyDoubleWidget,
        decimal = MyDecimalWidget,
        time = MyTimeWidget,
        date = MyDateWidget,
        datetime = MyDatetimeWidget,
        boolean = gluon.sqlhtml.BooleanWidget,
        ))

    def __init__(self, query, 
                 row_field, col_field, data_field, 
                 #query=None,
                 row_ids=None, col_ids=None,
                 row_represent=None, col_represent=None, 
                 data_represent=None, data_parse=None,
                 row_totals=False, col_totals=False,
                 extra_values=None,
                 next=None,
                 **attributes):
        '''Muestra una hoja de calculo para editar una relacion que representa
        una funcion (ref1,ref2) -> valor
        '''
        FORM.__init__(self, **attributes)
        if not (row_field.table == col_field.table == data_field.table):
            raise AttributeError
        self.table = row_field.table
        self.db    = row_field.db
        self.query = query or table
        self._id_field   = self.table.id
        self.row_field  = row_field
        self.col_field  = col_field
        self.data_field = data_field
#        self.row_totals = row_totals
        if row_totals:
            raise NotImplementedError
        self.col_totals = col_totals
        self.next = next
        if not row_represent:
            row_represent = row_field.represent or (lambda x:x)
        self.row_represent = row_represent
        if not col_represent:
            col_represent = col_field.represent or (lambda x:x)
        self.col_represent = col_represent
        if not data_represent:
            data_represent = data_field.represent or (lambda x:x)
        self.data_represent = data_represent
        if not data_parse:
            data_parse = lambda x:x #or self.data_field.default
        self.data_parse = data_parse

        if not extra_values:
            extra_values = {}
        self.extra_values = extra_values

        #En RWSheet, row_ids es necesariamente una lista de enteros
        if row_ids and not row_field.type.startswith('reference'):
            raise AttributeError('row_ids specified for row field not of referenfe type')
        if row_ids==None:
            row_ids   = [r[row_field] for r in self.db(query).select(row_field, groupby=row_field)]
        self.row_ids = row_ids
        self.row_names = [row_represent(rid) for rid in row_ids]

#        if col_ids and not col_field.type.startswith('reference'):
#            raise AttributeError('col_ids specified for col field not of reference type')
        if col_ids==None:
            col_ids   = [r[col_field] for r in self.db(query).select(col_field, groupby=col_field)]
        self.col_ids = col_ids
        self.col_names = [col_represent(cid) for cid in col_ids]

        self.prepare_components()

    def prepare_components(self):
        table_data = defaultdict(str)
        for q_row in self.db(self.query).select(self.row_field, self.col_field, self.data_field):
            r,c,d = q_row[self.row_field], q_row[self.col_field], q_row[self.data_field]
            table_data[(r,c)] = self.data_represent(d) or ''
        hs = [TH(n) for n in self.col_names]
        if self.col_totals:
            col_headers = THEAD(TR(TH(''),TH(TAG.strong(T('Totales'))),*hs))
        else:
            col_headers = THEAD(TR(TH(''),*hs))
        
        td_widget = self.td_widget()
        validator = self.data_field.requires
        
        def ts(rid):
            return [TD(td_widget(self.data_field,
                                  table_data[(rid, cid)], 
                                  _id='cell_%d_%d'%(rid,cid), 
                                  _name='cell_%d_%d'%(rid,cid),
                                  requires=validator)) 
                      for cid in self.col_ids]
        if self.col_totals:
            trows = [TR(TH(rname, _id='row_header_%d'%rid),
                        TD(SPAN('',_id = 'total_%d'%rid)),
                        *ts(rid)) 
                     for rid,rname in zip(self.row_ids,self.row_names) ]
        else:
            trows = [TR(TH(rname, _id='row_header_%d'%rid), *ts(rid)) 
                     for rid,rname in zip(self.row_ids,self.row_names) ]

        table = TABLE(col_headers, TBODY(*trows), **{'_class':'rwsheet', '_id':'rwsheet'})
        c = [INPUT(_type='hidden', _id='_row_ids', 
                   _name='_row_ids', _value=','.join(str(i) for i in self.row_ids)),
             INPUT(_type='hidden', _id='_col_ids', 
                   _name='_col_ids', _value=','.join(str(i) for i in self.col_ids)),
             INPUT(_type='hidden', _id='paste_into_cell',
                   _name='paste_into_cell', _value='nombre_0')]
        paste_helper = DIV(c, _class="hidden")

        self.components = [#css4rwsheet, 
                           #js4rwsheet,#, selected, shift_selected,
                           table, 
                           show_product_area, rwsheet_controls, 
                           paste_helper, #js_paste
                           ]
        response.files.append(URL('static', 'css/rsheet.css'))
        response.files.append(URL('static', 'js/jquery-migrate-1.2.1.js'))
        response.files.append(URL('static', 'js/jquery.fixheadertable.js'))
        response.files.append(URL('static', 'css/fixheadertable-base.css'))
        response.files.append(URL('static', 'css/rsheet.css'))
#        response.files.append(URL('static', 'jquery-ui/css/smoothness/jquery-ui-1.8.4.custom.css'))
        response.files.append(URL('static', 'js/rwsheet.js'))
        if self.col_totals:
            response.files.append(URL('static', 'js/rwsheet_col_totals.js'))

        
    def td_widget(self):
        field_type = self.data_field.type
        if field_type.startswith('decimal'):
            field_type = 'decimal'
        try:
            return self.widgets[field_type].widget
        except KeyError:
            raise AttributeError('RWSheet not implemented for fields of type %s'%field_type)

    def accepts(
        self,
        request_vars,
        session=None,
        formname=None,
        keepvalues=False,
        onvalidation=None,
        dbio=True,
        hideerror=False,
        detect_record_change=False,
        ):
        if request_vars.__class__.__name__ == 'Request':
            request_vars=request_vars.post_vars
        #Ojo: si han pulsado en discard changes, no escribes nada (y recargas la hoja)
        if request_vars.discard and self.next:
            redirect(self.next)
        if request_vars.discard or not request_vars.save:
            return False
        self.errors.clear()
        self.request_vars = Storage()
        self.request_vars.update(request_vars)
        self.session = session
        self.formname = formname
        self.keepvalues = keepvalues

        ret = FORM.accepts(
            self,
            request_vars,
            session,
            formname,
            keepvalues,
            onvalidation,
            hideerror=hideerror,
            )

        if ret and dbio:
            #No recuerdo si es que self.db(self.query).delete() no funciona
            # o que me preocupa que segun la query borre mas campos de los 
            # que yo queria
            for r in self.db(self.query).select(self._id_field):
                del self.table[r.id] 
            qs = []
            for k,v in request_vars.iteritems():
                if not k.startswith('cell'):
                    continue
                data = self.data_parse(v)
                if data:
                    _, rids, cids = k.split('_')
                    rid = int(rids)
                    cid = int(cids)
                    d = dict(self.extra_values)
                    d.update({self.row_field.name:rid, 
                              self.col_field.name:cid, 
                              self.data_field.name:data})
                    qs.append(d)
            self.table.bulk_insert(qs)
            #TODO: Se podria hacer mas eficiente actualizando los valores de las celdas
            #En vez de recrear todo el formulario, pero en fin, sera mas adelante
            self.prepare_components()
        
        return ret
        
###########################################################################################
#
#   Rchoose
#
###########################################################################################
#TODO
def rchoose(query, table, 

col_field, data_field, 
            row_ids=None, col_ids=None,
            row_represent=None, col_represent=None, data_represent=None):
    '''Choose multiple values from a table
    
    row_ids es una lista de ids de registros a usar en las filas, si el campo
        row_field es de tipo reference
    lo mismo para col_ids
    '''
    f = FORM(TABLE(TR(TD(T('Unidad')), TD(T('username')),
                  TD(T('Nombre')), TD(T('Teléfono')), TD(T('Email')),
                  TD(T('Eliminar del grupo'))),
               *[TR(TD(u.personaXgrupo.unidad),
                    TD(u.auth_user.username),
                    TD('%s %s'%(u.auth_user.first_name, u.auth_user.last_name)), 
                    TD(u.auth_user.telefono or T('Desconocido')), TD(u.auth_user.email),
                    TD(INPUT(_value='Eliminar', _type='checkbox', _name='del_%s'%u.auth_user.id) ))
                 for u in miembros]),
        INPUT(_type='submit', _name='submit', _value='Sacar del grupo a los usuarios seleccionados'))

    form = FORM()
    if not row_represent:
        row_represent = row_field.represent or (lambda x:x)
    if not col_represent:
        col_represent = col_field.represent or (lambda x:x)
    if not data_represent:
        try:
            data_represent = data_field.represent or (lambda x:x)
        except AttributeError:
            #A f.sum() or f.count() doesn't have a represent attribute
            data_represent = (lambda x:x)

    #En ROSheet, row_ids no es necesariamente una lista de enteros
    if not row_ids:
        row_ids   = [r[row_field] for r in db(query).select(row_field, groupby=row_field)]
    row_names = [row_represent(rid) for rid in row_ids]
    if not col_ids:
        col_ids   = [r[col_field] for r in db(query).select(col_field, groupby=col_field)]
    col_names = [col_represent(cid) for cid in col_ids]

    table_data = defaultdict(str)
    for q_row in db(query).select(row_field, col_field, data_field, groupby=row_field | col_field):
        r,c,d = q_row[row_field], q_row[col_field], q_row[data_field]
        table_data[(r,c)] = data_represent(d) or ''
    col_headers = THEAD(TR(TH(''),*[TH(n) for n in col_names]))
    trows = [TR(TH(row_represent(r)), 
               *[TD(table_data[(r, c)], _id='cell_%d_%d'%(j,k)) for k,c in enumerate(col_ids)]) 
            for j,r in enumerate(row_ids) ]
    response.files.append(URL('static', 'css/rsheet.css'))
    return DIV(#css4rosheet, #js4rosheet, selected, shift_selected,
                TABLE(col_headers, *trows, **{'_class':'rosheet'}))

