# -*- coding: utf-8 -*-
# (C) Copyright (C) 2012, 2013 Pablo Angulo
# This file is part of karakolas <karakolas.org>.

# karakolas is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as 
# published by the Free Software Foundation, either version 3 of the 
# License, or (at your option) any later version.

# karakolas is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU Affero General Public 
# License along with karakolas.  If not, see 
# <http://www.gnu.org/licenses/>.

#########################################################################

import copy
import datetime

#Mind this line: we use spanish only!!
#Atención: para simplificar, forzamos el idioma castellano
#(por el momento es la unica traduccion disponible)
#pero es posible quitar la linea y dejar la configuración del navegador
T.force('es-es')

PRECISION_DECIMAL = Decimal('0.01')

MESES = {1:T('Enero'), 2:T('Febrero'), 3:T('Marzo'), 4:T('Abril'), 
         5:T('Mayo'), 6:T('Junio'), 7:T('Julio'), 8:T('Agosto'), 
         9:T('Septiembre'), 10:T('Octubre'), 11:T('Noviembre'), 12:T('Diciembre'),}

AYUDA_PRODUCTOR = T('''
<h3>Abrir pedido</h3>
<p>bla bla bla</p>
<h3>Hacer el pedido</h3>
<p>bla bla bla</p>
<h3>Para el reparto</h3>
<p>bla bla bla</p>
<h3>Pagar</h3>
<p>bla bla bla</p>
''')

#TODO: Esto deberia ser configuracion de webadmin
CONTACTA_ADMIN  = T('Contacta con el administrador: pang@cancamusa.net')

db.define_table('grupo',
    Field('nombre','string', length=255, unique=True, writable=False),
    Field('direccion'),
    Field('descripcion', 'text'),
    Field('red', 'boolean', default=False, writable=False),
    Field('lista_de_espera', 'boolean', default=True),
    Field('discreto', 'boolean', default=False),
    Field('prueba', 'boolean', default=False, writable=False),
    format='%(nombre)s'
    )

db.define_table('grupoXred',
    Field('grupo', db.grupo),
    Field('red', db.grupo),
    )
db.grupoXred._mysql_index_command = 'ALTER TABLE grupoXred ADD UNIQUE (grupo,red)'

db.define_table('personaXgrupo',
    Field('grupo', db.grupo),
    Field('persona', db.auth_user),
    Field('unidad', 'integer', notnull=True),
#Las personas que se van del grupo no se borran, se desactivan 
#(para no estropear los informes antiguos)
    Field('activo', 'boolean', default=True)    
    )
db.personaXgrupo._mysql_index_command = 'ALTER TABLE personaXgrupo ADD UNIQUE (grupo, persona)'

def representa_usuario_2(uid):
    usuario = db.auth_user[uid]
    return '%s %s'%(usuario.first_name, 
                    usuario.last_name[0] if usuario.last_name else '')

def representa_usuario_3(uid, unidad=None, grupo=None):
    usuario = db.auth_user[uid]
    if not unidad:
        unidad  = db.personaXgrupo(persona=uid, grupo=grupo).unidad
    return XML('%d<br/>%s %s'%(unidad, usuario.first_name, 
                                usuario.last_name[0] if usuario.last_name else ''))

db.define_table('personaXlistaespera',
    Field('grupo', db.grupo),
    Field('persona', db.auth_user),
    Field('fecha_de_solicitud', 'datetime'),
    Field('comentario', 'string'),
#Las personas que se van de la lisa de espera se desactivan
#asi no se pierde la antiguedad en ningun caso (por ejemplo si se incorpora por error)
#y luego se borra
    Field('activo', 'boolean', default=True)    
    )
db.personaXlistaespera._mysql_index_command = 'ALTER TABLE personaXlistaespera ADD UNIQUE (grupo, persona)'

import string
import random
db.define_table('productor',
    Field('nombre','string', length=255, notnull=True),
    Field('email', 'string', length=255, requires=IS_EMPTY_OR(IS_EMAIL())),
    Field('grupo', db.grupo, writable=False),
    #El telefono es un string pq permite agrupar las cifras, poner prefijos y de todo.
    Field('telefono', 'string'),    
    Field('direccion','string'),
    Field('descripcion','text'),
    Field('info_pedido', 'text', default=AYUDA_PRODUCTOR),
    Field('formula_precio_final', 'string', default='precio_base'),
    Field('random_string', 'string', compute=lambda r:''.join(random.choice(string.letters) for i in xrange(32)), readable=False, writable=False),
    #Los dos campos siguientes son solo para guardar el ultimo valor usado en un pedido a este productor
#    Field('patron', 'string', default='nombre,precio,descripcion', writable=False),
    Field('reglas', 'text', default='', writable=False),
#    Field('automatico','boolean'),
#    Field('antelacion','?'),
    format='%(nombre)s')
#db.productor.email.requires = IS_EMAIL()
db.productor._mysql_index_command = 'ALTER TABLE productor ADD INDEX (grupo, nombre)'

def format_proxy(r):
    return 'Comentarios de %s a %s'%(r.grupo.nombre, r.productor.nombre)

db.define_table('proxy_productor',
    Field('productor', db.productor, requires = IS_IN_DB(db, db.productor, '%(nombre)s')),    
    Field('grupo', db.grupo, requires = IS_IN_DB(db, db.grupo, '%(nombre)s')),    
    Field('comentarios', 'text'),
    Field('aceptado', 'boolean', default=True),
    format=format_proxy)
db.proxy_productor._mysql_index_command = 'ALTER TABLE proxy_productor ADD UNIQUE (grupo, productor)'

db.define_table('categorias_productos',
    Field('nombre', 'string', length=255, unique=True),
    Field('codigo', 'integer', unique=True),
    format='%(nombre)s'
    )

db.define_table('producto',
    Field('nombre','string', length=255,notnull=True),
    Field('productor', db.productor,requires=IS_IN_DB(db,db.productor,'%(nombre)s')),#, readable=False, writable=False),
    Field('descripcion','string', length=2048, requires=IS_LENGTH(2048)),
    Field('categoria', db.categorias_productos, 
          requires=IS_IN_DB(db,db.categorias_productos,'%(nombre)s',
                        error_message=T('Por favor escoje una categoría'),
                        zero=T('Por favor escoje una categoría'))),
    #A granel si se pueden pedir fracciones (típicamente, se vende por kg)
    #Manojos, botellas, latas, pollos... no son a granel
    Field('granel','boolean', default=False),
    Field('activo','boolean', default=True),
    Field('temporada','boolean', default=True),
    #los campos siguientes se ponen por conveniencia, pero tienen un valor
    #distinto en cada pedido
    #el valor para el proximo pedido se guarda en productoXpedido
    #pero en el pedido "fantasma" del 1-1-1970
    #'precio_base' es el precio base, sin aplicar la formula
    Field('precio_base','decimal(9,2)',
          requires = IS_DECIMAL_IN_RANGE(-1e6,1e6,dot=',')),
    #'precio_final' es el precio despues de aplicar la formula
    Field('precio_final','decimal(9,2)',
          requires = IS_DECIMAL_IN_RANGE(-1e6,1e6,dot=',')),
    Field('destacado','boolean', default=False),
    Field('fila','integer', readable=False),
    Field('hoja','integer', readable=False),
    Field('columna_peticion','integer', readable=False),
    format='%(nombre)s')
#db.producto.productor.requires = IS_IN_DB(db, db.productor, '%(nombre)s')
#db.producto.categoria.requires = IS_IN_DB(db, db.categorias_productos, '%(nombre)s')
db.producto._mysql_index_command = 'ALTER TABLE producto ADD UNIQUE (productor, nombre)'

db.define_table('campo_extra',
    Field('productor', db.productor, requires=IS_IN_DB(db,db.productor,'%(nombre)s')),
    Field('nombre', 'string', length=255),
    Field('valor', 'decimal(9,2)'),
    format='%(nombre)s'
    )
db.campo_extra._mysql_index_command = 'ALTER TABLE campo_extra ADD INDEX (productor, nombre)'

NOMBRES_TIPOS_COLUMNA={
    'decimal(9,2)':T('Número'),
    'string': T('Texto'), 
    'boolean':T('Valor Sí o No')
}
#TIPOS_COLUMNA=['decimal(9,2)', 'string', 'boolean']
TIPOS_COLUMNA=NOMBRES_TIPOS_COLUMNA.keys()

def format_columna_extra(r):
#    return 'Pedido del %s'%r.fecha_reparto
    return '%s: %s'%(r.productor.nombre, r.nombre)

db.define_table('columna_extra',
    Field('productor', db.productor, requires=IS_IN_DB(db,db.productor,'%(nombre)s')),
    Field('nombre', 'string', length=255),
    Field('tipo', 'string', requires = IS_IN_SET(TIPOS_COLUMNA)),
    format=format_columna_extra
#    format='%(productor)s: (nombre)s'
    )
db.columna_extra._mysql_index_command = 'ALTER TABLE columna_extra ADD INDEX (productor, nombre)'

#Consistencia: columna.productor==producto.productor
db.define_table('valor_extra_decimal',
    Field('columna', db.columna_extra, requires=IS_IN_DB(db,db.columna_extra,'%(nombre)s')),
    Field('producto', db.producto),
    Field('valor', 'decimal(9,2)'),
    format='%(nombre)s'
    )
db.valor_extra_decimal._mysql_index_command = 'ALTER TABLE valor_extra_decimal ADD UNIQUE (columna, producto)'

#Consistencia: columna.productor==producto.productor
db.define_table('valor_extra_string',
    Field('columna', db.columna_extra, requires=IS_IN_DB(db,db.columna_extra,'%(nombre)s')),
    Field('producto', db.producto),
    Field('valor', 'string'),
    format='%(nombre)s'
    )
db.valor_extra_string._mysql_index_command = 'ALTER TABLE valor_extra_string ADD UNIQUE (columna, producto)'

#Consistencia: columna.productor==producto.productor
db.define_table('valor_extra_boolean',
    Field('columna', db.columna_extra, requires=IS_IN_DB(db,db.columna_extra,'%(nombre)s')),
    Field('producto', db.producto),
    Field('valor', 'boolean'),
    format='%(nombre)s'
    )
db.valor_extra_boolean._mysql_index_command = 'ALTER TABLE valor_extra_boolean ADD UNIQUE (columna, producto)'

TABLAS_VALORES_EXTRA = {
    'decimal(9,2)' : db.valor_extra_decimal,
    'string': db.valor_extra_string,
    'boolean'  : db.valor_extra_boolean
}

def format_fecha(fecha):
    y = fecha.year; m = fecha.month; d = fecha.day
    return '%d-%d-%d'%(d,m,y)

def format_pedido(r):
    return 'Pedido de %s del %s'%(r.productor.nombre, r.fecha_reparto)

db.define_table('pedido',
    Field('productor', db.productor, 
          requires = IS_IN_DB(db, db.productor, '%(nombre)s'), 
          readable=False, writable=False),
    Field('fecha_activacion', 'date', requires=IS_DATE(), 
          compute=lambda r:request.now.date),
    Field('info_cierre', 'text', default='El pedido se cierra el jueves XX de XXXXXX por la noche.'),
    Field('fecha_reparto', 'date', requires=IS_DATE()),
    Field('abierto', 'boolean', default=False),
    Field('usa_plantilla', 'boolean', default=False),
    Field('xls', 'upload', default=None, autodelete=True, uploadseparate=True),    
    #TODO: La formula_precio_pedido es por pedido, como las reglas
#    Field('formula_precio_final', 'string'),
    Field('reglas', 'text'),
    format=format_pedido
    )
db.pedido._mysql_index_command = 'ALTER TABLE pedido ADD INDEX (productor,fecha_reparto)'

FECHA_FANTASMA = datetime.datetime(1970,1,1)

db.define_table('campo_extra_pedido',
    Field('pedido', db.pedido, requires=IS_IN_DB(db,db.pedido,format_pedido)),
    Field('nombre', 'string', length=255),
    Field('valor', 'decimal(9,2)'),
    format='%(nombre)s'
    )
db.campo_extra_pedido._mysql_index_command = 'ALTER TABLE campo_extra_pedido ADD INDEX (pedido, nombre)'

db.define_table('columna_extra_pedido',
    Field('pedido', db.pedido, requires=IS_IN_DB(db,db.pedido,format_pedido)),
    Field('nombre', 'string', length=255),
    Field('tipo', 'string', length=64, requires = IS_IN_SET(TIPOS_COLUMNA)),
    format='%(productor)s: (nombre)s'
    )
db.columna_extra_pedido._mysql_index_command = 'ALTER TABLE columna_extra_pedido ADD INDEX (pedido, nombre)'

#Consistencia: columna.pedido.productor==producto.productor
db.define_table('valor_extra_decimal_pedido',
    Field('columna', db.columna_extra_pedido, requires=IS_IN_DB(db,db.columna_extra_pedido)),
#TODO: hacer este cambio de producto a productoXpedido
#    Field('productoXpedido', db.productoXpedido, requires=IS_IN_DB(db,db.productoXpedido)),
    Field('producto', db.producto, requires=IS_IN_DB(db,db.producto)),
    Field('valor', 'decimal(9,2)'),
    format='%(nombre)s'
    )
db.valor_extra_decimal_pedido._mysql_index_command = 'ALTER TABLE valor_extra_decimal_pedido ADD UNIQUE (columna, producto)'

#Consistencia: columna.pedido.productor==producto.productor
db.define_table('valor_extra_string_pedido',
    Field('columna', db.columna_extra_pedido, requires=IS_IN_DB(db,db.columna_extra_pedido)),
#    Field('productoXpedido', db.productoXpedido, requires=IS_IN_DB(db,db.productoXpedido)),
    Field('producto', db.producto, requires=IS_IN_DB(db,db.producto)),
    Field('valor', 'string'),
    format='%(nombre)s'
    )
db.valor_extra_string_pedido._mysql_index_command = 'ALTER TABLE valor_extra_string_pedido ADD UNIQUE (columna, producto)'

#Consistencia: columna.pedido.productor==producto.productor
db.define_table('valor_extra_boolean_pedido',
    Field('columna', db.columna_extra_pedido, requires=IS_IN_DB(db,db.columna_extra_pedido)),
#    Field('productoXpedido', db.productoXpedido, requires=IS_IN_DB(db,db.productoXpedido)),
    Field('producto', db.producto, requires=IS_IN_DB(db,db.producto)),
    Field('valor', 'boolean'),
    format='%(nombre)s'
    )
db.valor_extra_boolean_pedido._mysql_index_command = 'ALTER TABLE valor_extra_boolean_pedido ADD UNIQUE (columna, producto)'

TABLAS_VALORES_EXTRA_PEDIDO = {
    'decimal(9,2)' : db.valor_extra_decimal_pedido,
    'string'       : db.valor_extra_string_pedido,
    'boolean'      : db.valor_extra_boolean_pedido
}

EXTRA_PRODUCTO_2_PEDIDO = [(v, TABLAS_VALORES_EXTRA_PEDIDO[f]) 
                      for f,v in TABLAS_VALORES_EXTRA.iteritems()]

PEDIDOS_ABIERTOS   = 'abiertos'
PEDIDOS_CERRADOS   = 'cerrados'
PEDIDOS_HISTORICOS = 'historicos'
PEDIDOS_ACTIVOS = 'activos'
TIPO_PEDIDO = {PEDIDOS_ABIERTOS:T('abierto'),
               PEDIDOS_CERRADOS:T('cerrado'),
               PEDIDOS_HISTORICOS:T('histórico')}

#def format_proxy_pedido(r):
#    return 'Comentarios de %s a %s'%(r.grupo.nombre, r.productor.nombre)

db.define_table('proxy_pedido',
    Field('pedido', db.pedido, requires = IS_IN_DB(db, db.pedido, format_pedido)),    
    Field('grupo', db.grupo, requires = IS_IN_DB(db, db.grupo, '%(nombre)s')),    
    Field('comentarios', 'text', default='El pedido se cierra el jueves XX de XXXXXX por la noche.'),
    Field('fecha_reparto', 'date'),
    #incluyo este campo porque puede ser interesante poner comentarios para
    #justificar por que no se activa
    Field('activo', 'boolean', default=False)
#    format=format_proxy_pedido
    )
db.proxy_pedido._mysql_index_command = 'ALTER TABLE proxy_pedido ADD UNIQUE (grupo, pedido)'
    
#Consistencia: pedido.productor==producto.productor
db.define_table('productoXpedido',
    Field('pedido', db.pedido, 
          requires = IS_IN_DB(db, db.pedido, format_pedido)),
    Field('producto', db.producto, 
          requires = IS_IN_DB(db, db.producto, '%(nombre)s')),
    #'precio_base' es el precio base, sin aplicar la formula
    Field('precio_base','decimal(9,2)',
          requires = IS_DECIMAL_IN_RANGE(-1e6,1e6,dot=',')),
    #'precio_final' es el precio despues de aplicar la formula
    Field('precio_final','decimal(9,2)',
          requires = IS_DECIMAL_IN_RANGE(-1e6,1e6,dot=',')),
    Field('destacado','boolean', default=False),
    Field('fila','integer', readable=False),
    Field('hoja','integer', readable=False),
    Field('columna_peticion','integer', readable=False),
    )
db.productoXpedido._mysql_index_command = 'ALTER TABLE productoXpedido ADD UNIQUE (pedido, producto)'

db.define_table('mascara_producto',
    Field('productoXpedido', db.productoXpedido, requires = IS_IN_DB(db, db.productoXpedido)),    
    Field('grupo', db.grupo, requires = IS_IN_DB(db, db.grupo, '%(nombre)s')),
    Field('aceptado', 'boolean', default=True),
)
db.mascara_producto._mysql_index_command = 'ALTER TABLE mascara_producto ADD UNIQUE (grupo, productoXpedido)'

from decimal import Decimal
def compute_precio_total(r):
    if not r.cantidad:
        return Decimal(0)
    if isinstance(r.cantidad, str):
        q = r.cantidad.replace(',','.')
    else:
        q = r.cantidad
    return ((db.productoXpedido[r.productoXpedido]).precio_final)*Decimal(q)

db.define_table('item',
    Field('productoXpedido', db.productoXpedido,
          requires = IS_IN_DB(db, db.productoXpedido)),
    Field('unidad', 'integer'),
    #pensando en pedidos compartidos, donde el pedido no lleva la info del grupo
    Field('grupo', db.grupo),
    Field('cantidad', 'decimal(9,2)',
          requires = IS_EMPTY_OR(IS_DECIMAL_IN_RANGE(0,1e6,dot=',')),
          default=Decimal(0)),
    Field('precio_total', 'decimal(9,2)', compute=compute_precio_total)
    )
db.item._mysql_index_command = 'ALTER TABLE item ADD UNIQUE (grupo, productoXpedido, unidad)'

db.define_table('peticion',
    Field('productoXpedido', db.productoXpedido,
          requires = IS_IN_DB(db, db.productoXpedido)),
    Field('persona', db.auth_user),
    Field('grupo', db.grupo),
    Field('cantidad', 'decimal(9,2)',
          requires = IS_EMPTY_OR(IS_DECIMAL_IN_RANGE(0,1e6,dot=',')),
          default=Decimal(0)),
    Field('precio_total', 'decimal(9,2)', compute=compute_precio_total)
    )
db.peticion._mysql_index_command = 'ALTER TABLE peticion ADD UNIQUE (grupo, productoXpedido, persona)'

db.define_table('comentarioApeticion',
    Field('pedido', db.pedido),
    Field('persona', db.auth_user),
    Field('texto', 'text'),
    format='%(texto)s'
    )
db.comentarioApeticion._mysql_index_command = 'ALTER TABLE comentarioApeticion ADD UNIQUE (pedido, persona)'

#Por ahora no se usa...
db.define_table('comentarioAproducto',
    Field('producto', db.producto),
    Field('persona', db.auth_user),
    Field('fecha', 'datetime'),
    Field('texto', 'text'),
    )
db.comentarioAproducto._mysql_index_command = 'ALTER TABLE comentarioAproducto ADD UNIQUE (producto, persona)'

COBRO_DESGLOSADO        = 0
COBRO_AGRUPADO          = 1
REPARTO_FACIL           = 2
DESGLOSADO_POR_GRUPOS   = 3
COBRO_TOTALES           = 4
TIPOS_INFORME_GRUPO = {
    COBRO_DESGLOSADO:       T('Informe para cobro desglosado por personas'),
    COBRO_AGRUPADO:         T('Informe para cobro, agrupado por unidades'),
    REPARTO_FACIL:          T('Informe para reparto fácil'),
#    COBRO_TOTALES:          T('Precio a pagar a cada productor, por personas'),
}
TIPOS_INFORME_RED = {

    DESGLOSADO_POR_GRUPOS:  'Informe para la red, desglosado por grupos'
}
TIPOS_INFORME = dict(TIPOS_INFORME_GRUPO.items() + TIPOS_INFORME_RED.items())

db.define_table('informe',
    Field('fecha_reparto', 'date'),
    Field('grupo', db.grupo, requires = IS_IN_DB(db, db.grupo, '%(nombre)s')),    
    Field('pedidos', 'list:reference pedido'),
    Field('tipo_informe', 'integer', requires = IS_IN_SET(TIPOS_INFORME.keys())),
    Field('ruta', 'text'),
    Field('fecha_generacion', 'datetime', compute = lambda r:request.now),    
    #TODO: comentarios
    #Field('comentario', 'string'),
    )
db.informe._mysql_index_command = 'ALTER TABLE informe ADD INDEX (grupo, fecha_reparto)'

############# Actas y archivos del grupo ################
#db.define_table('fichero',
#    Field('nombre', unique=True),
#    Field('grupo', db.grupo),
#    Field('stamp', 'datetime', compute=lambda x:request.now),
#    Field('fichero', 'upload'),
#    format='%(nombre)s'
#    )

import datetime
now=datetime.datetime.now()

db.define_table('allfiles',
    Field('filename'),
    Field('filepath'),
    Field('parentpath'),
    Field('filetype'),
    Field('file','upload', autodelete=True, uploadseparate=True),
    Field('content','text'),
    Field('datecreated','datetime',default=now),
    Field('datemodified','datetime',default=now),
    Field('filesize','integer'),
#    Field('user',db.auth_user,default=None),
    Field('grupo',db.grupo)
    )
db.allfiles._mysql_index_command = 'ALTER TABLE allfiles ADD INDEX (grupo)'

db.define_table('paginas_especiales',
                Field('slug','string', length=255, writable=False, unique=True,
                      requires=(IS_SLUG(),IS_NOT_IN_DB(db,'paginas_especiales.slug'))),
                Field('title',default='',
                      requires=(IS_NOT_EMPTY(),IS_NOT_IN_DB(db,'paginas_especiales.title'))),
                Field('body','text',default=''),
                format = '%(slug)s')

db.define_table('textos_grupo',
    Field('grupo', db.grupo),
    Field('slug', 'string', length=255, writable=False, requires=IS_SLUG()),
    Field('texto', 'text', default='')
    )
db.textos_grupo._mysql_index_command = 'ALTER TABLE textos_grupo ADD UNIQUE (grupo,slug)'

def create_mysql_indices_run_once():
    for t in db:
        try:
            if hasattr(t,'_mysql_index_command'):
                db.executesql(t._mysql_index_command)
        except:
            print 'Error creating index on table ', t
            db.rollback()

