# -*- coding: utf-8 -*-
# (C) Copyright (C) 2012, 2013 Pablo Angulo
# This file is part of karakolas <karakolas.org>.

# karakolas is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as 
# published by the Free Software Foundation, either version 3 of the 
# License, or (at your option) any later version.

# karakolas is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU Affero General Public 
# License along with karakolas.  If not, see 
# <http://www.gnu.org/licenses/>.

#########################################################################

### utils ###
import re
CAMPO_EN_FILA_CSV = re.compile('[^,"]+|"(?:[^"]|"")+"|\A(?:""|)(?=,)|(?<=,)(?:""|)(?=,)|(?<=,)(?:""|)\Z')
def separa_por_coma(s):
    cs = CAMPO_EN_FILA_CSV.findall(s)
    return [c.replace('""','_DOUBLE COMMA_').replace('"','').replace('_DOUBLE COMMA_','"')
             if c!='""' else ''
             for c in cs]

def separa_lineas(s):
    ls1 = s.splitlines()
    ls2 = []
    xs  = []
    for l in ls1:
        comillas_impares = l.count('"')%2
        if xs and comillas_impares:
            xs.append(l)
            ls2.append('\n'.join(xs))
            xs = []
        elif xs or comillas_impares:
            xs.append(l)
        else:
            ls2.append(l)
    return ls2
### Redes y grupos ###

def redes_del_grupo(igrupo):
    s = set()
    for row in db((db.grupoXred.grupo==igrupo)) \
                .select(db.grupoXred.red):
        s.add(row.red)
        s.update(redes_del_grupo(row.red))
    return s
    

def mapa_grupo(igrupo):
    mapa = defaultdict(list)
    for row in db(db.personaXgrupo.grupo==igrupo)  \
                   .select(db.personaXgrupo.unidad, db.personaXgrupo.persona):
        mapa[row.unidad].append(row.persona)
    for v in mapa.values():
        v.sort()
    return mapa

def esta_en_red(ipersona, grupop):
    if db.personaXgrupo(persona=ipersona, grupo=grupop):
        return True
    return any(esta_en_red(ipersona, red.grupo)  
               for red in db(db.grupoXred.red==grupop) \
                   .select(db.grupoXred.grupo))

    grupos = db((db.grupoXred.red==ired) &
                (db.grupoXred.grupo==db.grupo.id)) \
               .select(db.grupo.ALL)

def collect_groups(ired):
    if db.grupo(ired).red:
        s = set()
        for subred in db((db.grupoXred.red==ired) &
                         (db.grupoXred.grupo==db.grupo.id)) \
                      .select(db.grupo.id):
            s.update(collect_groups(subred.id))
        return s
    else:
        return set((ired,))

def collect_users(ired):
    s = set()
    if db.grupo(ired).red:
        for subred in db((db.grupoXred.red==ired) &
                         (db.grupoXred.grupo==db.grupo.id)) \
                      .select(db.grupo.id):
            s.update(collect_users(subred.id))
            return s
    else:
        return set(row.id
                    for row in  db((db.personaXgrupo.grupo==ired) &
                       (db.personaXgrupo.persona==db.auth_user.id)) \
                    .select(db.auth_user.id))

### Utiles para permisos de acceso  ###

def grupo_del_productor(productor):
    try:
        grupo      = db.productor(productor).grupo
        return 'admins_%s'%grupo
    except:
        return 'webadmins'

def tiene_acceso_al_grupo(productor):
    try:
        grupo      = db.productor(productor).grupo
        return (auth.has_membership(role='admins_%s'%grupo) or
                 auth.has_membership(role='miembros_%s'%grupo) or
                 auth.has_membership(role='webadmins') )
    except:
        return auth.has_membership(role='webadmins')

def puede_leer_productos(iproductor, auth_code):
    try:
        productor = db.productor(iproductor)
        grupo     = productor.grupo
        random_string = productor.random_string
        return (auth.has_membership(role='admins_%s'%grupo) or
                 auth.has_membership(role='miembros_%s'%grupo) or
                 auth.has_membership(role='webadmins') or
                 random_string==auth_code)
    except:
        return auth.has_membership(role='webadmins')

def autoriza_pedido(vars, perms = 'rwa'):
    '''...
    
    perms pueden ser read, write, admin
    '''
    try:
        if 'grupo' in request.vars:
            igrupo = int(request.vars.grupo)
        else:
            ipedido = int(request.vars.pedido)
            igrupo  = db.pedido(ipedido).productor.grupo.id
    except:
        return False
    result = (auth.has_membership(role='admins_%s'%igrupo) or
              auth.has_membership(role='webadmins') )
    if 'a' not in perms:
        result = result or auth.has_membership(role='miembros_%s'%igrupo)
    return result

def grupo_para_pedir(pedido):
    try:
        productor = db.pedido(pedido).productor
        grupo     = db.productor(productor).grupo
        return 'miembros_%s'%grupo
    except:
        return 'webadmins'

def grupo_del_pedido(pedido):
    try:
        productor = db.pedido(pedido).productor
        grupo     = db.productor(productor).grupo
        return 'admins_%s'%grupo
    except:
        return 'webadmins'

def puede_pedir(pedido):
    try:
        ipersona  = auth.user.id
        productor = db.pedido(pedido).productor
        grupop    = db.productor(productor).grupo
        return esta_en_red(ipersona, grupop)            
    except:
        return auth.has_membership('webadmins')

### Productos ###

def ordena_categorias(cats):
    '''Ordena las categorias por codigo
    '''
    #¿se puede hacer mas eficiente?
    rows = db(db.categoria.id.belongs(cats)) \
            .select(db.categoria.id,
                    orderby=db.categoria.codigo)
    return [row.id for row in rows]

def ordena_productos(prods):
    '''Ordena los productos por el orden canonico, que ahora es categoria y 
    luego nombre por orden alfabetico
    '''
    #¿se puede hacer mas eficiente?
    rows = db(db.producto.id.belongs(prods) &
              (db.producto.categoria==db.categorias_productos.id)) \
            .select(db.producto.id,
                    orderby=db.categorias_productos.codigo|db.producto.nombre)
    return [row.id for row in rows]

### Pedidos ###

def pedidos_del_dia(fecha_reparto, igrupo):
    repartos = db((db.pedido.fecha_reparto==fecha_reparto) &
                  (db.pedido.productor==db.productor.id) &
                  (db.productor.grupo==igrupo)) \
                .select(db.pedido.ALL)
    repartosc = db((db.proxy_pedido.fecha_reparto==fecha_reparto) &
                   (db.proxy_pedido.pedido==db.pedido.id) &
                   (db.proxy_pedido.grupo==igrupo) &
                   (db.proxy_pedido.activo==True)) \
                .select(db.pedido.ALL)
    return (repartos, repartosc)

def dame_pedidos_por_fecha(igrupo, q, todos_los_coordinados=False):
    today = datetime.date.today()
    query_base = ((db.pedido.productor==db.productor.id) &
                  (db.productor.grupo==igrupo))
    query_c_base = ((db.proxy_pedido.pedido==db.pedido.id) &
                    (db.pedido.productor==db.productor.id) &
                    (db.proxy_productor.productor==db.productor.id) &
                    (db.proxy_productor.grupo==igrupo) &
                    (db.proxy_productor.aceptado==True) &
                    (db.proxy_pedido.grupo==igrupo)) 
    if q==PEDIDOS_ABIERTOS:
        query   = query_base & (db.pedido.abierto==True)                 
        query_c = query_c_base & (db.pedido.abierto==True) 
    elif q==PEDIDOS_CERRADOS:
        query   = query_base & ((db.pedido.abierto==False) & 
                                (db.pedido.fecha_reparto>=today))
        query_c = query_c_base & ((db.pedido.abierto==False) & 
                                  (db.pedido.fecha_reparto>=today))
    elif q==PEDIDOS_HISTORICOS:
        query   = query_base & ((db.pedido.abierto==False) & 
                                (db.pedido.fecha_reparto<today))
        query_c = query_c_base & ((db.pedido.abierto==False) & 
                                  (db.pedido.fecha_reparto<today))
    if not todos_los_coordinados:
        query_c &= (db.proxy_pedido.activo==True)
    s = db(query) \
        .select(db.pedido.ALL, db.productor.nombre, db.productor.id)
    s2 = db(query_c)  \
          .select(db.pedido.ALL, db.proxy_pedido.fecha_reparto,
                  db.productor.nombre, db.productor.id,
                  orderby=db.pedido.fecha_reparto)

    fechas = set()
    for row in s:
        fechas.add(row.pedido.fecha_reparto)
    for row in s2:
        fechas.add(row.proxy_pedido.fecha_reparto)
    pedidos = dict((f, (list(), list())) for f in fechas)
    for row in s:
        pedidos[row.pedido.fecha_reparto][0].append(row)
    for row in s2:
        pedidos[row.proxy_pedido.fecha_reparto][1].append(row)
    ls = pedidos.items()
    ls.sort(reverse=True)
    return [(format_fecha(f), rs) for f,rs in ls]

def mueve(tabla1, valores1, tabla2, valores2, keys=None):
    '''Mueve el resultado de tabla1, parametrizado con valores1 a 
    tabla2, parametrizado con valores2
    
    Sólo se mueven los campos comunes a ambas tablas
    Comprueba si el registro en tabla2 ya existe y en ese caso lo actualiza
    Devuelve un diccionario con la correspondencia entre los ids de los 
    registros correspondientes
    
    Elimina los registros de tabla2 determinados por valores2
    '''
    q1 = reduce(lambda a,b:(a & b) , 
                    [tabla1[t]==v 
                     for t,v in valores1.iteritems() ])
    q2 = reduce(lambda a,b:(a & b) , 
                    [tabla2[t]==v 
                     for t,v in valores2.iteritems() ])
    db(q2).delete()

    ignored_fields = ['id'] + valores1.keys()
    fields = set(tabla2.fields).intersection(tabla1.fields).difference(ignored_fields)
    mapa = {}
    for row in db(q1).select(tabla1.ALL):
        d = dict((f, row[f]) for f in fields)
        d.update(valores2)
        mapa[row.id] = tabla2.insert(**d)

    return mapa

#def add_dicts(*ds):
#    from itertools import chain
#    return dict(chain(*[d.iteritems() for d in ds]))

def mueve_productos_al_pedido(ipedido):
    '''Define los productoXpedido para un pedido, volcando todos los productos
    marcados como activos
    '''
    iproductor = db.pedido[ipedido].productor
    v1 = {'productor':iproductor}
    v2 = {'pedido':ipedido}

    smart_update(db.productoXpedido, 
                 db.productoXpedido.pedido==ipedido,
                 [{'pedido':ipedido, 'producto':r.id,
                   'precio_final':r.precio_final, 'precio_base':r.precio_base,
                   'destacado':r.destacado, 
                   'fila':r.fila, 'hoja':r.hoja, 'columna_peticion':r.columna_peticion}
                   for r in db((db.producto.activo==True) &
                               (db.producto.temporada==True) &
                               (db.producto.productor==iproductor))\
                            .select(db.producto.ALL)],
                ['producto', 'pedido']
                )

    mueve(db.campo_extra, v1, db.campo_extra_pedido, v2, keys=['nombre'])
    map = mueve(db.columna_extra, v1, db.columna_extra_pedido, v2)
    for col1,col2 in map.iteritems():
        tipo = db.columna_extra[col1].tipo
        t1 = TABLAS_VALORES_EXTRA[tipo]
        t2 = TABLAS_VALORES_EXTRA_PEDIDO[tipo]
        mueve(t1, {'columna':col1}, t2, {'columna':col2})

def mueve_productos_al_pedido_fantasma(iproductor):
    pedido_fantasma = db.pedido(productor=iproductor,fecha_reparto=FECHA_FANTASMA)
    mueve_productos_al_pedido(pedido_fantasma.id)

def actualiza_productos_desde_el_pedido(ipedido):
    '''Inverso del anterior, carga los campos de un productoXpedido
    en la tabla producto. Util para usar una ERTable
    '''
    iproductor = db.pedido[ipedido].productor
    v1 = {'pedido':ipedido}
    v2 = {'productor':iproductor}

    smart_update(db.producto, 
                 db.producto.productor==iproductor,
                 [{'productor':iproductor, 'id':r.producto,
                   'precio_final':r.precio_final, 'precio_base':r.precio_base,
                   'destacado':r.destacado, 
                   'fila':r.fila, 'hoja':r.hoja, 'columna_peticion':r.columna_peticion,
                   'activo':True, 'temporada':True}
                   for r in db(db.productoXpedido.pedido==ipedido)\
                            .select(db.productoXpedido.ALL)],
                ['id'],
                action=lambda s:s.update(temporada=False) 
                )

    mueve(db.campo_extra_pedido, v1, db.campo_extra, v2, keys=['nombre'])
    map = mueve(db.columna_extra_pedido, v1, db.columna_extra, v2)
    for col1,col2 in map.iteritems():
        tipo = db.columna_extra_pedido[col1].tipo
        t1 = TABLAS_VALORES_EXTRA_PEDIDO[tipo]
        t2 = TABLAS_VALORES_EXTRA[tipo]
        mueve(t1, {'columna':col1}, t2, {'columna':col2})

def actualiza_productos_desde_pedido_fantasma(iproductor):
    pedido_fantasma = db.pedido(productor=iproductor,fecha_reparto=FECHA_FANTASMA)
    actualiza_productos_desde_el_pedido(pedido_fantasma.id)

def actualiza_precios_peticion(ipedido):
    '''Actualiza los precios de peticion y item cuando se cambia el precio
    de roductos en un pedido ya abierto
    '''
    for r in db(db.productoXpedido.pedido==ipedido) \
        .select(db.productoXpedido.id, db.productoXpedido.precio_final):
        db(db.peticion.productoXpedido==r.id) \
            .update(precio_total=db.peticion.cantidad*r.precio_final)
        db(db.item.productoXpedido==r.id) \
            .update(precio_total=db.item.cantidad*r.precio_final)

def dame_valores_extra(ipedido):
    extra = defaultdict(list)
    for n,t in TABLAS_VALORES_EXTRA_PEDIDO.iteritems():
        extras = db((db.columna_extra_pedido.pedido==ipedido) &
                    (db.columna_extra_pedido.tipo==n) &
                    (t.columna==db.columna_extra_pedido.id)) \
                 .select(t.producto, t.valor, db.columna_extra_pedido.nombre)
        for row in extras:
            #TODO: procesa el valor (ej, Si/no en vez de True/None, o cambia punto por coma decimal)
            extra[row[t].producto].append('%s: %s'%(row.columna_extra_pedido.nombre, row[t].valor))
    return extra

### Mascaras ##

def query_mask(igrupo):
    '''Uso: unir con & a otra query
    
    Por ej: solo los productos del pedido que el grupo ha aceptado

    (db.productoXpedido.pedido==ipedido) & query_mask(igrupo)
    '''
    return ((db.mascara_producto.productoXpedido==db.productoXpedido.id) &
             (db.mascara_producto.grupo==igrupo) &
             (db.mascara_producto.aceptado==True))        

def activa_productos_coordinados(ipedido, igrupo):
    productos_posibles = db(db.productoXpedido.pedido==ipedido)\
                         .select(db.productoXpedido.ALL)
    for row in productos_posibles:
        if not db.mascara_producto(productoXpedido=row.id, grupo=igrupo):
            db.mascara_producto.insert(productoXpedido=row.id, grupo=igrupo, aceptado=True)

### Peticiones y items ###

def peticion_a_item(ipedido, igrupo, unidad=None):
    #Borra items antiguos
    q_items = ((db.item.grupo==igrupo) &
               (db.item.productoXpedido==db.productoXpedido.id) &
               (db.productoXpedido.pedido==ipedido))
    if unidad:
        q_items &= (db.item.unidad==unidad)
    d = db(q_items) \
        .select(db.item.id)
    ls = [row.id for row in d]
    db(db.item.id.belongs(ls)).delete()

    #Nuevos items asumiendo que llegan todas las peticiones
    cantidad_total = db.peticion.cantidad.sum()
    q_pets = ((db.peticion.persona==db.auth_user.id) &
              (db.peticion.grupo==igrupo) &
              (db.personaXgrupo.persona==db.auth_user.id) &
              (db.personaXgrupo.grupo==igrupo) &
              (db.peticion.productoXpedido==db.productoXpedido.id) &
              (db.productoXpedido.pedido==ipedido))
    if unidad:
        q_pets &= (db.personaXgrupo.unidad==unidad)
    s = db(q_pets) \
        .select(cantidad_total, 
                db.personaXgrupo.unidad, 
                db.productoXpedido.id,
                groupby=db.personaXgrupo.unidad|db.productoXpedido.id)
    for row in s:
        db.item.insert(grupo=igrupo, 
                       productoXpedido=row.productoXpedido.id,
                       unidad=row.personaXgrupo.unidad,
                       cantidad=row[cantidad_total])

def recoge_y_ordena_item(ipedidos, igrupo):
    cantidades = {}
    unidades = set()
    productos  = {}
    for ipedido in ipedidos:
        rows = db((db.item.grupo==igrupo) & 
                  (db.item.productoXpedido==db.productoXpedido.id) &
                  (db.productoXpedido.pedido==ipedido) &
                  (db.productoXpedido.producto==db.producto.id) ) \
               .select(db.item.ALL, 
                       db.productoXpedido.precio_final,
                       db.producto.ALL) 
        cs = defaultdict(lambda : defaultdict(Decimal))
        for row in rows:
            if row.item.cantidad>0:
                productos[row.producto.id] = (row.producto.nombre, row.producto.granel,
                                              row.productoXpedido.precio_final)
                unidades.add(row.item.unidad)
                cs[row.producto.id][row.item.unidad] += row.item.cantidad
                
        cantidades[ipedido] = cs
    lunidades = list(unidades)
    lunidades.sort()
    return lunidades, productos, cantidades

def recoge_y_ordena_peticion(ipedidos, igrupo):
    cantidades = {}
    personas = {}
    productos  = {}
    for ipedido in ipedidos:
        rows = db((db.peticion.productoXpedido==db.productoXpedido.id) &
                  (db.productoXpedido.pedido==ipedido)         &
                  (db.peticion.persona==db.auth_user.id) & 
                  (db.peticion.grupo==igrupo) &
                  (db.personaXgrupo.persona==db.auth_user.id) & 
                  (db.personaXgrupo.grupo==igrupo) & 
                  (db.productoXpedido.producto==db.producto.id) ) \
               .select(db.peticion.cantidad, db.peticion.precio_total, 
                       db.productoXpedido.precio_final,
                       db.auth_user.id,
                       db.personaXgrupo.unidad,
                       db.producto.ALL) 
        cs = defaultdict(lambda : defaultdict(Decimal))
        for row in rows:
            if row.peticion.cantidad>0:
                productos[row.producto.id] = (row.producto.nombre, row.producto.granel,
                                              row.productoXpedido.precio_final)
                personas[(row.personaXgrupo.unidad, row.auth_user.id)] = \
                    (representa_usuario_2(row.auth_user.id))
                cs[row.producto.id][row.auth_user.id] += row.peticion.cantidad
                
        cantidades[ipedido] = cs
    lpersonas = [(u,id,un) for ((u,id), un) in personas.iteritems()]
    lpersonas.sort()
    return lpersonas, productos, cantidades

def recoge_y_ordena_item_por_grupos(ipedidos, ired):
    cantidades = {}
    grupos = set()
    productos  = {}
    totales = db.item.cantidad.sum()
    for ipedido in ipedidos:
        rows = db((db.item.grupo==db.grupoXred.grupo) &
                  (db.grupoXred.red==ired) &
                  (db.item.productoXpedido==db.productoXpedido.id) &
                  (db.productoXpedido.pedido==ipedido) &
                  (db.productoXpedido.producto==db.producto.id) ) \
               .select(db.item.grupo, totales,
                       db.productoXpedido.precio_final,
                       db.producto.ALL,
                       groupby=db.item.grupo|db.productoXpedido.producto) 
        cs = defaultdict(lambda : defaultdict(Decimal))
        for row in rows:
            row_totales = row[totales]
            if row_totales>0:
                productos[row.producto.id] = (row.producto.nombre, row.producto.granel,
                                              row.productoXpedido.precio_final)
                grupos.add(row.item.grupo)
                cs[row.producto.id][row.item.grupo] += row_totales
                
        cantidades[ipedido] = cs
    lgrupos = list(grupos)
    lgrupos.sort()
    return lgrupos, productos, cantidades

def crea_pedido_fantasma(iproductor):
    r = db.pedido(productor=iproductor, fecha_reparto=FECHA_FANTASMA)
    if not r:
        rid = db.pedido.insert(productor=iproductor, fecha_reparto=FECHA_FANTASMA, abierto=False)
        mueve_productos_al_pedido(rid)

import re
AMBITOS = re.compile('^PRODUCTOR$|^CAMPOS EXTRA$|^COLUMNAS EXTRA$|^PRODUCTOS$', flags=re.MULTILINE)
#CAMPOS  = re.compile('([^\,]*),"(.+?[^"])";')
CAMPOS  = re.compile('([^,"]+),("(?:[^"]|"")+"|"");')
N_COLUMNAS_PRODUCTOS = 7
def importar_productor(igrupo, csv_data):
    '''Importa productor con todos sus productos en el grupo indicado
    el formato del archivo csv es el generado por gestion_pedidos/exportar_productos.csv
    '''
    _, dproductor, dcampos_extra, dcolumnas_extra, dproductos = AMBITOS.split(csv_data)
    d = dict( (c,v.replace('""','_DOUBLE COMMA_').replace('"','').replace('_DOUBLE COMMA_','"')
            if v!='""' else '')
            for c,v in
            CAMPOS.findall(dproductor.lstrip().replace('\n',';')))
    d['grupo'] = igrupo
#    raise Exception, d
    iproductor = db.productor.insert(**d)
    for row in dcampos_extra.strip().splitlines()[1:]:
        nombre,valor = row.split(',')
        db.campo_extra.insert(nombre=nombre,valor=valor,productor=iproductor)
    columnas_extra = {}
    for row in dcolumnas_extra.strip().splitlines()[1:]:
        nombre,tipo = separa_por_coma(row)
        icolumna = db.columna_extra.insert(nombre=nombre,tipo=tipo,productor=iproductor)
        columnas_extra[nombre] = (tipo, icolumna)
    ls = separa_lineas(dproductos.strip())
    fs = ls[0].split(',')
    producto_campos = [fs[i].split('.')[1] for i in xrange(N_COLUMNAS_PRODUCTOS)]
    for row in ls[1:]:
        cs = separa_por_coma(row)
        #TODO: cambia del nombre de las categorias al id, quiza sea mas consistente incluir
        #el codigo unspsc al exportar...
        d = dict(zip(producto_campos, cs[:N_COLUMNAS_PRODUCTOS]))
        codigo = int(cs[N_COLUMNAS_PRODUCTOS])
        cid = db.categorias_productos(codigo=codigo)
        if not cid:
            #TODO: busca la categoria existente mas aproximada!
            if 50000000<codigo<60000000:
                cid = db.categorias_productos(codigo=50000000)
            else:
                cid = db.categorias_productos(codigo=-1)
        d['categoria'] = cid
        d['productor'] = iproductor
        iproducto = db.producto.insert(**d)
        for i in xrange(N_COLUMNAS_PRODUCTOS+1,len(fs)):
            nombre = fs[i].split('.')[0]
            tipo, icolumna = columnas_extra[nombre]
            tipo = tipo.split('(')[0]
            db['valor_extra_%s'%tipo].insert(producto=iproducto,columna=icolumna,valor=cs[i])

    crea_pedido_fantasma(iproductor)
