# -*- coding: utf-8 -*-
# (C) Copyright (C) 2013, 2013 Pablo Angulo
# This file is part of karakolas <karakolas.org>.

# karakolas is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as 
# published by the Free Software Foundation, either version 3 of the 
# License, or (at your option) any later version.

# karakolas is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU Affero General Public 
# License along with karakolas.  If not, see 
# <http://www.gnu.org/licenses/>.

#########################################################################
#Objetivo: adivinar la categoria a partir del nombre, usando las clasificaciones 
#previas que ha hecho el usuario. Usamos el metodo Naive Bayes, pero con algun 
#apaño y cuidando la distribucion a priori
#La documentacion de scikits.learn es muy clara:
#http://scikit-learn.org/stable/modules/naive_bayes.html
#pero a la hora de usarlo, me parecio mas facil escribir el codigo yo, y 
#controlar mejor la prior despues de un rato probando y pensando, me quede con 
#la multinomial
#http://scikit-learn.org/stable/modules/naive_bayes.html#multinomial-naive-bayes

import numpy
import pickle
import os.path

TAKE_MODE_THRESHOLD   = 1.25
RECOMPUTE_ON_NEW_DATA = 1.10
CAT_NINGUNA = db.categorias_productos(codigo=-1).id
CAT_ALIMENTOS = db.categorias_productos(codigo=50000000).id

class Bayes_Filter(object):
    LIDSTONE_ALPHA = 0.5
    def __init__(self):
        try:
            self.pickle_load()
            self.recalcula_si_nec()
        except IOError, EOFError:
            self.restart()
            self.pickle_dump()

    def pickle_load(self):
        self.M, self.prior, self.dns, self.dcats, self.data_size = pickle.load(
            open(os.path.join(request.folder,'private','bayes.pickle') ,'r'))
    
    def pickle_dump(self):
        pickle.dump((self.M, self.prior, self.dns, self.dcats, self.data_size) ,
            open(os.path.join(request.folder,'private','bayes.pickle') ,'w'),
            protocol=2)

    def restart(self):
        cats = [r.id for r in db(db.categorias_productos.codigo>0).select()]
        dcats = dict((c,j) for j,c in enumerate(cats))
#        dcats_invs = dict((x,y) for y,x in dcats.iteritems())
        ncats = len(dcats)
        prior = [1.0]*ncats

        rows = db(~db.producto.categoria.belongs(CAT_NINGUNA, CAT_ALIMENTOS)) \
              .select(db.producto.nombre, db.producto.categoria)
        self.data_size = len(rows)
        ns = set()
        for r in rows:
            for w in r.nombre.split():
                if len(w)>2:
                    ns.add(w.strip().lower())
            prior[dcats[r.categoria]] += 1
        total = sum(prior)
        self.prior = numpy.matrix(frec/total for frec in prior)

        #Alternativa mas rapida para prior, pero hace mas probables las categorias con productos cuyo nombre tiene mas palabras
        #prior = [sum(M[j,:]) for j in xrange(ncats)]

        dns = dict((w,j) for j,w in enumerate(ns) if len(w)>2)
        nns = len(dns)

        M = self.LIDSTONE_ALPHA*numpy.ones((ncats,nns), dtype=numpy.float)
        for r in rows:
            for w in r.nombre.split():
                if len(w)>2:
                    M[dcats[r.categoria],dns[w.strip().lower()]] += 1

        self.dcats = dcats
        self.dns   = dns
        self.M     = M
        self.prior = prior

    def recalcula_si_nec(self):
        N = db(~db.producto.categoria.belongs(CAT_NINGUNA, CAT_ALIMENTOS)).count()
        if self.data_size*RECOMPUTE_ON_NEW_DATA < N:
            self.pickle_dump()
            
    def posterior(self, xs, y, first_guess=None):
        if first_guess:
            pass #TODO
        p = self.prior[y]/sum(self.M[y,:])**(len(xs))
        for x in xs:
            p *= self.M[y,x]
        return p

    def adivina_categorias(self,nombre_producto, first_guess=None):
        words = [w.strip().lower() for w in nombre_producto.split()]
        wcols = [self.dns.get(w,-1) for w in words if len(w)>2]
        wcols = [w for w in wcols if w>=0]
        ls = [(self.posterior(wcols,crow),cid)  for cid,crow in self.dcats.iteritems()]
        ls.sort(reverse=True)
        first_prob, first_cat = ls[0]
        second_prob, _        = ls[1]
#        print [(db.categorias_productos(cid).nombre,p) for p,cid in ls[:5]]
        if first_prob>TAKE_MODE_THRESHOLD*second_prob:
            return first_cat
        else:
            return CAT_NINGUNA

bayes = Bayes_Filter()
