# -*- coding: utf-8 -*-
# (C) Copyright (C) 2012, 2013 Pablo Angulo
# This file is part of karakolas <karakolas.org>.

# karakolas is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as 
# published by the Free Software Foundation, either version 3 of the 
# License, or (at your option) any later version.

# karakolas is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU Affero General Public 
# License along with karakolas.  If not, see 
# <http://www.gnu.org/licenses/>.

#########################################################################

TEMPLATE_VERTICAL = r'''\documentclass[a4paper]{article}
\usepackage[scaled=FONT_SCALE]{helvet}
\renewcommand*\familydefault{phv}
\usepackage[utf8]{inputenc}
%\usepackage[top=1cm,bottom=1cm,left=1cm,right=1cm]{geometry}
\usepackage[margin=1cm]{geometry}
\usepackage{longtable}
\usepackage{eurosym}
\usepackage[table]{xcolor}
\usepackage{soul}

\begin{document}
\rowcolors{1}{lightgray}{white}

tabla_latex
\end{document}
'''

TEMPLATE_APAISADO = r'''\documentclass[landscape, a4paper]{article}
\usepackage[scaled=FONT_SCALE]{helvet}
\renewcommand*\familydefault{phv}
\usepackage[utf8]{inputenc}
\usepackage[top=1cm,bottom=1cm,left=1cm,right=1cm]{geometry}
\usepackage{longtable}
\usepackage{eurosym}
\usepackage[table]{xcolor}
\usepackage{soul}

%\setlength{\oddsidemargin}{0in}        % default=0in
%\setlength{\textwidth}{9in}        % default=9in
\addtolength{\textwidth}{40pt}        % default=9in
\addtolength{\topmargin}{-20pt}        % default=0.20in
\addtolength{\textheight}{40pt}        % default=5.15in
\addtolength{\leftmargin}{-20pt}        % default=???

\begin{document}
\rowcolors{1}{lightgray}{white}

tabla_latex
\end{document}
'''
#Excepto por los dos primeros, no podria explicar el valor del resto
#de las constantes; las he puesto tras ensayo y error
PAGE_WIDTH       = 21.0
PAGE_HEIGHT      = 29.7
PROD_WIDTH       = 2.0
CELL_WIDTH_LARGE = 0.8
CELL_WIDTH       = 0.5
CELL_WIDTH_SHORT = 0.3
MULTIPLIER       = 1.8
#MARGIN           = 1.0
FONT_SCALE_BASE  = 0.6
FONT_SCALE_BIS   = 0.4

def latex_escape(s):
    return s.replace('%','\%').replace('_','\\_')

def pprint_p(x):
    return '%s'%x.quantize(PRECISION_DECIMAL)

def denormalize(x):
    sign, digits, expo = x.normalize().as_tuple()
    if expo>=0:
        return Decimal( (sign, digits + (0,)*expo, 0 ) )
    return x.normalize()
    
def pprint_c(x):
    return '%s'%denormalize(x.quantize(PRECISION_DECIMAL))

def tabla_totales(ipedidos, igrupo, apaisado, es_red=False):
    total = db.item.precio_total.sum()
    if es_red:
        igrupos = collect_groups(igrupo)
        q = (db.item.grupo.belongs(igrupos) &
             (db.item.productoXpedido==db.productoXpedido.id) &
             db.productoXpedido.pedido.belongs(ipedidos))
    else:
        q = ((db.item.grupo==igrupo) &
             (db.item.productoXpedido==db.productoXpedido.id) &
             db.productoXpedido.pedido.belongs(ipedidos))
    totales = db(q) \
              .select(total, 
                      db.productoXpedido.pedido, 
                      groupby=db.productoXpedido.pedido,
                      orderby=db.productoXpedido.pedido)
    productores = [row.productoXpedido.pedido.productor  for row in totales]
    pedidos     = [latex_escape(p.nombre) for p in productores]
    telefonos   = [str(p.telefono or '') for p in productores]
    pedidos.append('Total'); telefonos.append('')
    precios = [row[total] for row in totales]
    precios.append(sum(precios))
    #TODO: Asumo que caben en la hoja
    #¿por que no transponerla???
    width = PAGE_HEIGHT if apaisado else PAGE_WIDTH
    ncols = len(ipedidos) + 1
    cell_width = width/(ncols*2)
    ls = [(r'\begin{longtable}{' + 
          ('|p{%.2fcm}'%cell_width)*ncols + '|}\n') ]
    ls.append(r'\hline %s \\'%' & '.join(pedidos))
    ls.append(r'\hline %s \\'%' & '.join(telefonos))
    ls.append(r'\hline %s \\'%' & '.join('%s'%p for p in precios))
    ls.append(r'\hline\end{longtable}')
    return '\n'.join(ls)


def crear_informe_completo(ipedidos, igrupo, 
                           columnas_por_tabla=20,
                           apaisado=True,
                           **kwds):
    ###Adquisicion de datos
    mapag   = mapa_grupo(igrupo)
    unidades, productos_i, cantidades_i = recoge_y_ordena_item(ipedidos, igrupo)
    personas, productos_p, cantidades_p = recoge_y_ordena_peticion(ipedidos, igrupo)

    ###Merge de los items por unidades y las peticiones por personas
    #Podria ocurrir que se asignase pedido a una unidad que no lo pidio
    #O que una persona pidiese algo pero no le llegara nada
    spersonas = set(p for (_,p,_) in personas)
    us = set(unidades)
    us.update(u for u,_,_ in personas)
    unidades = list(us)
    unidades.sort()
    u2ps = dict((u, [p for p in mapag[u] if p in spersonas]) for u in unidades)
    username = dict((id,un) for (_,id,un) in personas)

    #Puede que un producto que se pidio no llegue, o que llegue uno que se pidio 
    iproductos = set(productos_i.keys() + productos_p.keys() )

    ordered_prods = db(db.producto.id.belongs(iproductos)) \
                    .select(db.producto.id, 
                            orderby=db.producto.categoria|db.producto.nombre)
    lproductos = [(row.id, productos_i.get(row.id) or 
                           productos_p.get(row.id))
                  for row in ordered_prods]

    productos = dict(lproductos)
    productos_ids = productos.keys()

    ###Preparar las columnas, como si no fuera nec cortar la tabla en partes
    columnas  = []
#    for unidad,miembros in u2ps.iteritems():
    for unidad in unidades:
        miembros = u2ps[unidad]
        colgroup = []
        for p in miembros:
            colgroup.append(username[p])
        colgroup.append(T('Total').xml())
        colgroup.append(T('Precio').xml())
        columnas.append((unidad, T('Unidad %s').xml()%unidad, colgroup))
    ncolumnas = sum(len(colgroup) for colgroup in columnas)

    datos   = {}
    totales = {}
    gran_total = dict((unidad,Decimal()) for unidad in unidades)
    for ipedido in ipedidos:
        tabla = []
        totales_pedido = dict((unidad,Decimal()) for unidad in unidades)
        ci_pedido = cantidades_i[ipedido]
        cp_pedido = cantidades_p[ipedido]
        prods = set(ci_pedido.keys() + cp_pedido.keys())
        prods = ordena_productos(prods)

        for pid in prods:
            _, granel,precio_f = productos[pid]
            fila = []
            ci_prod = ci_pedido[pid]
            cp_prod = cp_pedido[pid]
#            for unidad,miembros in u2ps.iteritems():                
            for unidad in unidades:
                miembros = u2ps[unidad]
                cantidad_ps = Decimal()
                for p in miembros:
                    cantidad = cp_prod[p]
                    cantidad_ps += cantidad
                    fila.append('%s'%pprint_c(cantidad) if cantidad else '')
                cantidad_i = ci_prod[unidad]
                precio     = cantidad_i*precio_f
                if cantidad_i or cantidad_ps:
                    fila.append('%s'%pprint_c(cantidad_i)
                                if (cantidad_ps==cantidad_i) else
                                (r'\st{' + '%s' + '} ' + '%s')%(pprint_c(cantidad_ps), pprint_c(cantidad_i)))
                else:
                    fila.append('')
                fila.append('%s'%pprint_p(precio) if precio else '')
                totales_pedido[unidad] += precio
            tabla.append((pid, fila))

        datos[ipedido] = tabla
        totales[ipedido] = totales_pedido
        for unidad in u2ps:
            gran_total[unidad] += totales_pedido[unidad]

    ###Preparamos la tabla final
    width = PAGE_HEIGHT if apaisado else PAGE_WIDTH
    total_width = PROD_WIDTH + CELL_WIDTH*(columnas_por_tabla+1)
    scale       = width / (total_width*MULTIPLIER)
    widths =dict(product_width='%.2f'%(PROD_WIDTH*scale),  
                 normal_cell_width='%.2f'%(CELL_WIDTH*scale),  
                ) 
    ls = []
    icol_start = 0
    icol_end   = 0
    while columnas:
        cols = [columnas.pop(0)]
        lcols = len(cols[0][2])
        while columnas and (lcols + len(columnas[0][2]) <= columnas_por_tabla):
            cols.append(columnas.pop(0))
            lcols += len(cols[-1][2])
        icol_end += lcols
        #Tabla con los grupos que estan en cols
#        ls.append(r'\begin{longtable}{|p{3cm}' + '||p{.5cm}|p{1cm}'*lcols + '||}\n')
        ls.append((r'\begin{longtable}{|p{%(product_width)scm}|p{%(normal_cell_width)scm}||' + 
                   '||'.join('|'.join('p{%(normal_cell_width)scm}' for _ in range(len(colgroup)))
#                   '%s p{.7cm}|p{1cm}'%('p{0.7cm}|'*(len(colgroup) - 2) )
                            for _,_,colgroup in cols) + 
                   '||}\n')%widths)
        cabecera1   = r'\hiderowcolors\hline & & %s\\'% \
                       ' & '.join(r'\multicolumn{%d}{c||}{%s}'%
                                 (len(colgroup), unidad)
                                  for _,unidad,colgroup in cols)
        cabecera2   = r'\hline & precio unitario & %s\\\showrowcolors '% \
                       ' & '.join(' & '.join(colgroup)
                                  for _,_,colgroup in cols)
        for ipedido in ipedidos:
            pedido = db.pedido[ipedido]
            totales_pedido = totales[ipedido]
            ls.append(r'\hline\multicolumn{%d}{|c|}{\textbf{%s}}\\'%
                      (lcols+2, latex_escape(pedido.productor.nombre)))
            ls.append(cabecera1)
            ls.append(cabecera2)

            for pid,fila in datos[ipedido]:
                pnombre,_,pprecio = productos[pid]
                ls.append(r'\hline %s & %s & %s \\'%(
                    latex_escape(pnombre),
                    pprint_p(pprecio),
                    ' & '.join(fila[icol_start:icol_end])) )

            ls.append(r'\hiderowcolors\hline Total Unidad & & %s\\'% \
                       ' & '.join(r'\multicolumn{%d}{c||}{%s}'%
                                 (len(colgroup), pprint_p(totales_pedido[iunidad]))
                                  for iunidad,_,colgroup in cols)
                    )

        ls.append(r'\hiderowcolors\hline Gran Total Unidad & & %s\\'% \
                   ' & '.join(r'\multicolumn{%d}{c||}{%s}'%
                             (len(colgroup), pprint_p(gran_total[iunidad]))
                              for iunidad,_,colgroup in cols)
                )
        ls.append(r'\hline\end{longtable}')
        icol_start = icol_end
    tabla_latex         = '\n'.join(ls)
    tabla_latex_totales = tabla_totales(ipedidos, igrupo, apaisado)

    if apaisado:
        latex = TEMPLATE_APAISADO
    else:
        latex = TEMPLATE_VERTICAL
    return latex.replace('FONT_SCALE', '%.2f'%(scale*FONT_SCALE_BIS)) \
                 .replace('tabla_latex','%s'%tabla_latex + tabla_latex_totales)

def crear_informe_agrupado(ipedidos, igrupo, 
                           columnas_por_tabla=20,
                           apaisado=True,
                           **kwds):
    unidades_por_tabla = columnas_por_tabla//2
    columnas_por_tabla = unidades_por_tabla*2
    pedidos = [db.pedido[id] for id in ipedidos]
    unidades, productos, cantidades = recoge_y_ordena_item(ipedidos, igrupo)
    productos_ids = productos.keys()
    productos_ids.sort()

    width = PAGE_HEIGHT if apaisado else PAGE_WIDTH
    total_width = (PROD_WIDTH + 
                   (CELL_WIDTH + CELL_WIDTH_SHORT)*columnas_por_tabla/2)
    scale       = width / (total_width*MULTIPLIER)
    widths =dict(product_width='%.2f'%(PROD_WIDTH*scale),  
                 normal_cell_width='%.2f'%(CELL_WIDTH*scale),  
                 short_cell_width='%.2f'%(CELL_WIDTH_SHORT*scale)
                ) 
    
    from math import ceil
    ntablas      = int(ceil(float(len(unidades))/unidades_por_tabla))
    ls           = []
    for k in range(ntablas):
        lunidades    = unidades[k*unidades_por_tabla:(k+1)*unidades_por_tabla]
        nunidades    = len(lunidades)
        gran_total  = dict(zip(lunidades, [Decimal()]*nunidades))

        row_unidades = r'\hiderowcolors\hline & %s\\\showrowcolors '% \
                       ' & '.join(r'\multicolumn{2}{c||}{%d}'%unidad  
                                  for unidad in lunidades)
        ls.append((r'\begin{longtable}{|p{%(product_width)scm}' + 
                    '||p{%(short_cell_width)scm}|p{%(normal_cell_width)scm}'*nunidades + 
                    '||}\n')%widths)

        for pedido in pedidos:
            precios_totales = dict(zip(lunidades, [Decimal()]*nunidades))
            cants = cantidades[pedido.id]
            ls.append(r'\hline\multicolumn{%d}{|c|}{\textbf{%s}}\\'%
                      (2*nunidades + 1, latex_escape(pedido.productor.nombre)) )
            ls.append(row_unidades)

            prods = ordena_productos(cants.keys())

#            for pid,peticiones in cants.iteritems():
            for pid in prods:
                peticiones = cants[pid]
                pnombre, granel, pprecio = productos[pid]
                pnombre = latex_escape(pnombre)
                pprecio = pprecio

#                if granel:
                cs = [peticiones.get(unidad, Decimal()) for unidad in lunidades]
                ls.append(r'\hline %s & '%pnombre + 
                          ' & '.join('%s & %s\\geneuro'%( pprint_c(c), pprint_p(pprecio*c) )
                                     if c>0 else ' & ' 
                                     for c in cs) +
                          r' \\')
#                else:
#                    cs = [peticiones.get(unidad, Decimal()) for unidad in lunidades]
#                    ls.append(r'\hline %s & '%pnombre + 
#                              ' & '.join('%d & %.2f\\geneuro'%(c, pprecio*c) 
#                                         if c>0 else ' & ' 
#                                         for c in cs) +
#                              r' \\')
                for unidad,c in zip(lunidades, cs):
                    precios_totales[unidad] += c*pprecio

            for unidad in lunidades:
                gran_total[unidad] += precios_totales[unidad]

            #Totales
            ls.append(r'\hline Total & '+ 
                          ' & '.join('  & %s\\geneuro'%pprint_p(precios_totales[unidad])
                                     for unidad in lunidades) +
                          r' \\')    
        #Grand Totales
        if len(ipedidos)>1:
            ls.append(r'\hline\multicolumn{%d}{|c|}{\textbf{%s}}\\'%(2*nunidades + 1, T('Gran Total')))
            ls.append(r'\hline %s & '%T('Gran Total')+ 
                          ' & '.join('  & %s\\geneuro'%pprint_p(gran_total[unidad])
                                     for unidad in lunidades) +
                          r' \\')    
        ls.append(r'\hline\end{longtable}')
#        ls.append(r'\newpage')
    tabla_latex = '\n'.join(ls)
    tabla_latex_totales = tabla_totales(ipedidos, igrupo, apaisado)

    if apaisado:
        latex = TEMPLATE_APAISADO
    else:
        latex = TEMPLATE_VERTICAL
    return latex.replace('FONT_SCALE', '%.2f'%(scale*FONT_SCALE_BIS)) \
                 .replace('tabla_latex','%s'%tabla_latex + tabla_latex_totales)

def crear_informe_reparto(ipedidos, igrupo, 
                          columnas_por_tabla=20,
                          apaisado=True,
                          **kwds):
    pedidos = [db.pedido[id] for id in ipedidos]
    unidades, productos, cantidades = recoge_y_ordena_item(ipedidos, igrupo)
    productos_ids = productos.keys()
    productos_ids.sort()

    width = PAGE_HEIGHT if apaisado else PAGE_WIDTH
    total_width = (PROD_WIDTH + 
                   CELL_WIDTH_SHORT*columnas_por_tabla)
    scale       = width / (total_width*MULTIPLIER)
    widths =dict(product_width='%.2f'%(PROD_WIDTH*scale),  
                 short_cell_width='%.2f'%(CELL_WIDTH_SHORT*scale)
                ) 
    
    from math import ceil
    ntablas      = int(ceil(float(len(unidades))/columnas_por_tabla))
    ls           = []
    for k in range(ntablas):
        lunidades    = unidades[k*columnas_por_tabla:(k+1)*columnas_por_tabla]
        nunidades    = len(lunidades)

        row_unidades = r'\hiderowcolors\hline & %s\\ \showrowcolors '% \
                       ' & '.join('%d'%unidad for unidad in lunidades)
        ls.append((r'\begin{longtable}{|p{%(product_width)scm}' + 
                  '|p{%(short_cell_width)scm}'*nunidades + '|}\n')%widths)

        for pedido in pedidos:
            cants = cantidades[pedido.id]
            ls.append(r'\hline\multicolumn{%d}{|c|}{\textbf{%s}}\\'%
                      (nunidades+1, latex_escape(pedido.productor.nombre)) )
            ls.append(row_unidades)

            prods = ordena_productos(cants.keys())

#            for pid,peticiones in cants.iteritems():
            for pid in prods:
                peticiones = cants[pid]
                pnombre, granel, _ = productos[pid]
                pnombre = latex_escape(pnombre)
#                if granel:
                cs = [peticiones.get(unidad, Decimal()) for unidad in lunidades]
                ls.append(r'\hline %s & '%pnombre + 
                          ' & '.join('%s'%pprint_c(c) 
                                     if c>0 else ' ' 
                                     for c in cs) +
                          r' \\')
#                else:
#                    cs = [peticiones.get(unidad, Decimal()) for unidad in lunidades]
#                    ls.append(r'\hline %s & '%pnombre + 
#                              ' & '.join('%d '%c 
#                                         if c>0 else ' ' 
#                                         for c in cs) +
#                              r' \\')

        ls.append(r'\hline\end{longtable}')
#        ls.append(r'\newpage')
    tabla_latex = '\n'.join(ls)
    if apaisado:
        latex = TEMPLATE_APAISADO
    else:
        latex = TEMPLATE_VERTICAL
    return latex.replace('FONT_SCALE', '%.2f'%(scale*FONT_SCALE_BIS)) \
                 .replace('tabla_latex','%s'%tabla_latex)    

def crear_informe_por_grupos(ipedidos, ired, 
                             columnas_por_tabla=20,
                             apaisado=True,
                             **kwds):
    pedidos = [db.pedido[int(p)] for p in ipedidos]
    grupos_id, productos, cantidades = recoge_y_ordena_item_por_grupos(ipedidos, ired)

    grupos    = [db.grupo(gid) for gid in grupos_id]

    productos_ids = productos.keys()
    productos_ids.sort()

    width = PAGE_HEIGHT if apaisado else PAGE_WIDTH
    total_width = (PROD_WIDTH + 
                   (CELL_WIDTH + CELL_WIDTH_LARGE)*columnas_por_tabla/2)
    scale       = width / (total_width*MULTIPLIER)
    widths = dict(product_width='%.2f'%(PROD_WIDTH*scale),  
                  normal_cell_width='%.2f'%(CELL_WIDTH*scale),  
                  large_cell_width='%.2f'%(CELL_WIDTH_LARGE*scale) )
    
    from math import ceil
    ntablas      = int(ceil(float(len(grupos))/columnas_por_tabla))
    ls           = []
#    ls.append('%s, %s!!'%(ntablas, grupos ))
    for k in range(ntablas):
        lgrupos    = grupos[k*columnas_por_tabla:(k+1)*columnas_por_tabla]
        lgrupos_id = [row.id for row in lgrupos]
        ngrupos    = len(lgrupos)
        row_grupos = r'\hline & %s\\'% \
                       ' & '.join(r'\multicolumn{2}{c||}{%s}'%latex_escape(row.nombre)
                                  for row in lgrupos)
        ls.append((r'\begin{longtable}{|p{%(product_width)scm}' + 
                    '||p{%(large_cell_width)scm}|p{%(large_cell_width)scm}'*ngrupos + 
                    '||}\n')%widths)
        for pedido in pedidos:
            precios_totales = dict(zip(lgrupos_id, [Decimal()]*ngrupos))
            cants = cantidades[pedido.id]
            ls.append(r'\hline\multicolumn{%d}{|c|}{\textbf{%s}}\\'%
                      (2*ngrupos + 1, latex_escape(pedido.productor.nombre)) )
            ls.append(row_grupos)

            prods = ordena_productos(cants.keys())

            for pid in prods:
                peticiones = cants[pid]
                pnombre, granel, pprecio = productos[pid]
                pnombre = latex_escape(pnombre)
                cs = [peticiones.get(gid, Decimal()) for gid in lgrupos_id]
                for gid, cantidad in zip(lgrupos_id, cs):
                    precios_totales[gid] += cantidad*pprecio

                ls.append(r'\hline %s & '%pnombre + 
                          ' & '.join('%s & %s\\geneuro'%(pprint_c(c), pprint_p(pprecio*c) )
                                     if c>0 else ' & ' 
                                     for c in cs) +
                          r' \\')
                
            #Totales
            ls.append(r'\hline Total & '+ 
                          ' & '.join('  & %s\\geneuro'%pprint_p(precios_totales[gid])
                                     for gid in lgrupos_id) +
                          r' \\')    
        ls.append(r'\hline\end{longtable}')
#        ls.append(r'\newpage')
    tabla_latex = '\n'.join(ls)
    tabla_latex_totales = tabla_totales(ipedidos, ired, apaisado, es_red=True)

    if apaisado:
        latex = TEMPLATE_APAISADO
    else:
        latex = TEMPLATE_VERTICAL
    return latex.replace('FONT_SCALE', '%.2f'%(scale*FONT_SCALE_BASE)) \
                 .replace('tabla_latex','%s'%tabla_latex + tabla_latex_totales)

#Lo dejo aparcado
def crear_informe_totales(ipedidos, igrupo, personas_por_columna=6):
    pedidos = [db.pedido[id] for id in ipedidos]
    cantidad_total = db.item.cantidad.sum()
    def ha_pedido(persona):
        return  db((db.item.productoXpedido==db.productoXpedido.id) &
                    (db.productoXpedido.pedido.belongs(pedidos))  &
                    (db.item.persona==persona) ).count()
    personas = [row for row in db((db.personaXgrupo.grupo==igrupo) &
                                  (db.personaXgrupo.persona==db.auth_user.id)) \
                                   .select(db.auth_user.ALL, db.personaXgrupo.unidad,
                                   orderby=db.personaXgrupo.unidad|db.auth_user.username)
                          if ha_pedido(row.auth_user.id)]
    personas_id  = [row.auth_user.id for row in personas]
    from math import ceil
    ntablas      = int(ceil(float(len(personas))/personas_por_columna))
    ls           = []
    for k in range(ntablas):
        lpersonas     = personas[k*personas_por_columna:(k+1)*personas_por_columna]
        lpersonas_id  = [row.auth_user.id for row in lpersonas]
        npersonas     = len(lpersonas)
        row_unidades = r'\hline & %s\\'% \
                       ' & '.join(r'\multicolumn{2}{l||}{%d}'%row.personaXgrupo.unidad  
                                  for row in lpersonas)
        row_personas = r'\hline & %s\\'% \
                       ' & '.join(r'\multicolumn{2}{l||}{%s}'%latex_escape(row.auth_user.username)
                                  for row in lpersonas)
        row_vacia    = r'\hline & %s\\'% \
                       ' & '.join(r'\multicolumn{2}{l||}{}' for row in lpersonas)
        ls.append(r'\begin{longtable}{|p{3cm}' + '||p{.7cm}|p{1cm}'*npersonas + '||}\n\hiderowcolors\n')
        ls.append(row_unidades)
        ls.append(row_personas)
        gran_total = dict(zip(lpersonas_id, [0.0]*npersonas))
        for pedido in pedidos:
            precios_totales = dict(zip(lpersonas_id, [0.0]*npersonas))
            ls.append(r'\hline\multicolumn{%d}{|c|}{\textbf{%s}}\\'%
                      (2*npersonas + 1, latex_escape(pedido.productor.nombre)) )

            for row in db(db.productoXpedido.pedido==pedido.id) \
                       .select(db.productoXpedido.ALL):
                #Si nadie ha pedido el producto, no se pone la linea
                ct = db((db.item.productoXpedido==db.productoXpedido.id) &
                        (db.productoXpedido.pedido==pedido)         &
                        (db.productoXpedido.producto==row.producto) ) \
                     .select(cantidad_total,
                             groupby=db.item.productoXpedido)
                if not ct or not ct.first()[cantidad_total]:
                    continue
                #Seguimos
                producto = db.producto[row.producto]
                pprecio  = float(row.precio)
                pnombre  = latex_escape(producto.nombre)
                granel   = producto.granel
                cantidades = {}
                for pid in lpersonas_id:
                    q = db.item(productoXpedido=row.id, persona=pid)
                    cantidad = float(q.cantidad or 0) if q else 0.0
                    cantidades[pid]      =  cantidad
                    precios_totales[pid] += cantidad*pprecio

            for pid in lpersonas_id:
                gran_total[pid] += precios_totales[pid]
            ls.append(r'\hline Total & '+ 
                          ' & '.join('  & %.2f\\geneuro'%precios_totales[pid] 
                                     for pid in lpersonas_id) +
                          r' \\')    
            ls.append(row_vacia)
            ls.append(row_vacia)

        ls.append(r'\hline Gran total & '+ 
                      ' & '.join('  & %.2f\\geneuro'%gran_total[pid] 
                                 for pid in lpersonas_id) +
                      r' \\')    
        ls.append(row_vacia)
        ls.append(row_vacia)

        ls.append(r'\hline\end{longtable}')
#        ls.append(r'\newpage')
    return '\n'.join(ls)

CREAR_INFORME = {
    COBRO_DESGLOSADO:        crear_informe_completo,
    COBRO_AGRUPADO:          crear_informe_agrupado,
    REPARTO_FACIL:           crear_informe_reparto,
#    COBRO_TOTALES:           crear_informe_totales,
    DESGLOSADO_POR_GRUPOS:   crear_informe_por_grupos
}


def generar_informe(tipo_informe, ipedidos, fecha_reparto, igrupo, **kwds):
    pedidos = [db.pedido[id] for id in ipedidos]
    table_func = CREAR_INFORME[tipo_informe]

    latex_content = table_func(ipedidos, igrupo, **kwds)

    #Write tex file
    str_fecha = format_fecha(fecha_reparto)
    rutatex = 'cobro_%s_%s_%s.tex' %(fecha_reparto, 
                                     '_'.join('%s'%p.id for p in pedidos),
                                     str(datetime.datetime.now()).replace(' ','_').replace(':','.') )
    outfile = codecs.EncodedFile(open(rutatex, 'w'), 'utf-8', 'utf-8')
    outfile.write(latex_content)
    outfile.close()

    wd = os.path.join(os.getcwd(), 'applications', request.application, 'static', 'informes')
    command = ['pdflatex', '-halt-on-error', '-output-directory', wd, rutatex]
    process = subprocess.Popen(command, bufsize=-1, 
                               stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    stdoutdata, stderrdata = process.communicate()
    exit_code = process.returncode
    if exit_code:
        raise Exception, latex_content
    base, _ = os.path.splitext(rutatex)
    rutapdf = base + '.pdf'
    rutaaux = base + '.aux'
    rutalog = base + '.log'
    os.remove(rutatex)
    os.remove(os.path.join(wd, rutaaux))
    os.remove(os.path.join(wd, rutalog))

    db.informe.insert(tipo_informe=tipo_informe,
                      grupo=igrupo,
                      pedidos=ipedidos,
                      fecha_reparto=fecha_reparto,   ruta=rutapdf)


