# -*- coding: utf-8 -*-
# (C) Copyright (C) 2012-2014 Pablo Angulo
# This file is part of karakolas <karakolas.org>.

# karakolas is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as 
# published by the Free Software Foundation, either version 3 of the 
# License, or (at your option) any later version.

# karakolas is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU Affero General Public 
# License along with karakolas.  If not, see 
# <http://www.gnu.org/licenses/>.

#########################################################################

TEMPLATE_VERTICAL = r'''\documentclass[a4paper]{article}
\usepackage[scaled=FONT_SCALE]{helvet}
\renewcommand*\familydefault{phv}
\usepackage[utf8]{inputenc}
%\usepackage[top=1cm,bottom=1cm,left=1cm,right=1cm]{geometry}
\usepackage[margin=1cm]{geometry}
\usepackage{longtable}
\usepackage{eurosym}
\usepackage[table]{xcolor}
\usepackage{soul}

\begin{document}
\rowcolors{1}{lightgray}{white}

tabla_latex
\end{document}
'''

TEMPLATE_APAISADO = r'''\documentclass[landscape, a4paper]{article}
\usepackage[scaled=FONT_SCALE]{helvet}
\renewcommand*\familydefault{phv}
\usepackage[utf8]{inputenc}
\usepackage[top=1cm,bottom=1cm,left=1cm,right=1cm]{geometry}
\usepackage{longtable}
\usepackage{eurosym}
\usepackage[table]{xcolor}
\usepackage{soul}

%\setlength{\oddsidemargin}{0in}        % default=0in
%\setlength{\textwidth}{9in}        % default=9in
\addtolength{\textwidth}{40pt}        % default=9in
\addtolength{\topmargin}{-20pt}        % default=0.20in
\addtolength{\textheight}{40pt}        % default=5.15in
\addtolength{\leftmargin}{-20pt}        % default=???

\begin{document}
\rowcolors{1}{lightgray}{white}

tabla_latex
\end{document}
'''
#Excepto por los dos primeros, no podria explicar el valor del resto
#de las constantes; las he puesto tras ensayo y error
PAGE_WIDTH       = 21.0
PAGE_HEIGHT      = 29.7
PROD_WIDTH       = 2.0
CELL_WIDTH_LARGE = 0.8
CELL_WIDTH       = 0.5
CELL_WIDTH_SHORT = 0.3
MULTIPLIER       = 1.8
#MARGIN           = 1.0
FONT_SCALE_BASE  = 0.6
FONT_SCALE_BIS   = 0.4

def latex_escape(s):
    return s.replace('%','\%').replace('_','\\_')

def pprint_p(x):
    return '%s'%x.quantize(PRECISION_DECIMAL)

def denormalize(x):
    sign, digits, expo = x.normalize().as_tuple()
    if expo>=0:
        return Decimal( (sign, digits + (0,)*expo, 0 ) )
    return x.normalize()
    
def pprint_c(x):
    return '%s'%denormalize(x.quantize(PRECISION_DECIMAL))

def latex_table(cabecera, filas, columnas, data, 
                ancho_cols = None ,columnas_por_tabla=20, apaisado=True, 
                **kwds):

    ###Preparamos la tabla final
    width = PAGE_HEIGHT if apaisado else PAGE_WIDTH
    total_width = PROD_WIDTH + CELL_WIDTH*columnas_por_tabla
    scale       = width / (total_width*MULTIPLIER)
    widths =dict(product_width='%.2f'%(PROD_WIDTH*scale),  
                 normal_cell_width='%.2f'%(CELL_WIDTH*scale),  
                )
    ls = []
    icol_start = 0
    icol_end   = 0
    while columnas:
        cols = [columnas.pop(0)]
        lcols = len(cols[0][2])
        while columnas and (lcols + len(columnas[0][2]) <= columnas_por_tabla):
            cols.append(columnas.pop(0))
            lcols += len(cols[-1][2])
        icol_end += lcols
        #Tabla con los grupos que estan en cols
#        ls.append(r'\begin{longtable}{|p{3cm}' + '||p{.5cm}|p{1cm}'*lcols + '||}\n')
        ls.append((r'\begin{longtable}{|p{%(product_width)scm}||' + 
                   '||'.join('|'.join('p{%(normal_cell_width)scm}' for _ in range(len(colgroup)))
#                   '%s p{.7cm}|p{1cm}'%('p{0.7cm}|'*(len(colgroup) - 2) )
                            for _,_,colgroup in cols) + 
                   '||}\n')%widths)
        cabecera1   = r'\hiderowcolors\hline & %s\\'% \
                       ' & '.join(r'\multicolumn{%d}{c||}{%s}'%
                                 (len(colgroup), unidad)
                                  for _,unidad,colgroup in cols)
        cabecera2   = r'\hline & %s\\\showrowcolors '% \
                       ' & '.join(' & '.join(colgroup)
                                  for _,_,colgroup in cols)
        for ipedido in ipedidos:
            pedido = db.pedido[ipedido]
            totales_pedido = totales[ipedido]
            ls.append(r'\hline\multicolumn{%d}{|c|}{\textbf{%s}}\\'%
                      (lcols+1, latex_escape(pedido.productor.nombre)))
            ls.append(cabecera1)
            ls.append(cabecera2)

            for pid,fila in datos[ipedido]:
                ls.append(r'\hline %s & %s \\'%(latex_escape(productos[pid][0]), ' & '.join(fila[icol_start:icol_end])) )

            ls.append(r'\hiderowcolors\hline Total Unidad & %s\\'% \
                       ' & '.join(r'\multicolumn{%d}{c||}{%s}'%
                                 (len(colgroup), pprint_p(totales_pedido[iunidad]))
                                  for iunidad,_,colgroup in cols)
                    )

        ls.append(r'\hiderowcolors\hline Gran Total Unidad & %s\\'% \
                   ' & '.join(r'\multicolumn{%d}{c||}{%s}'%
                             (len(colgroup), pprint_p(gran_total[iunidad]))
                              for iunidad,_,colgroup in cols)
                )
        ls.append(r'\hline\end{longtable}')
        icol_start = icol_end
    tabla_latex         = '\n'.join(ls)
    tabla_latex_totales = tabla_totales(ipedidos, igrupo, apaisado)

    if apaisado:
        latex = TEMPLATE_APAISADO
    else:
        latex = TEMPLATE_VERTICAL
    return latex.replace('FONT_SCALE', '%.2f'%(scale*FONT_SCALE_BIS)) \
                 .replace('tabla_latex','%s'%tabla_latex + tabla_latex_totales)

